package at.qubic.api.crypto;

import com.sun.jna.Memory;
import com.sun.jna.Native;
import lombok.extern.slf4j.Slf4j;
import at.qubic.api.crypto.jna.QubicCryptoLib;

@Slf4j
public class QubicFourQ implements FourQ, SchnorrQubic {

    private final QubicCryptoLib lib;

    public QubicFourQ() {
        this("fourq-qubic");
    }

    public QubicFourQ(String libName) {
        log.info("Using fourq library: [{}].", libName);
        lib = Native.load(libName, QubicCryptoLib.class);
    }

    @Override
    public byte[] ecc_mul_fixed(byte[] in) {
        final Memory p = new Memory(64);
        final byte[] out = new byte[32];
        lib.ecc_mul_fixed(in, p);
        lib.encode(p, out);
        return out;
    }

    @Override
    public void ecc_mul_fixed(byte[] in, byte[] out) {
        final Memory p = new Memory(64);
        lib.ecc_mul_fixed(in, p);
        lib.encode(p, out);
    }

    @Override
    public byte[] sign(byte[] subSeed, byte[] publicKey, byte[] digest) {
        assert subSeed.length >= 32;
        byte[] signature = new byte[64];
        lib.sign(subSeed, publicKey, digest, signature);
        return signature;
    }

    @Override
    public byte[] signWithNonceK(byte[] input, byte[] publicKey, byte[] digest) {
        assert input.length >= 64;
        byte[] signature = new byte[64];
        lib.signWithNonceK(input, publicKey, digest, signature);
        return signature;
    }

    @Override
    public boolean verify(byte[] publicKey, byte[] messageDigest, byte[] signature) {
        return lib.verify(publicKey, messageDigest, signature);
    }

}
