package at.qubic.api.crypto.jna;

import com.sun.jna.Library;
import com.sun.jna.Pointer;

public interface QubicCryptoLib extends Library {

    // FourQ's basic element definitions and point representations
    //
    // typedef digit_t felm_t[NWORDS_FIELD]; // Datatype for representing 128-bit field elements (NWORDS_FIELD is 2 on amd64)
    // typedef felm_t f2elm_t[2]; // Datatype for representing quadratic extension field elements
    //
    // typedef struct { f2elm_t x; f2elm_t y; } point_affine; // Point representation in affine coordinates.
    // typedef point_affine point_t[1];
    //
    // Java:
    //
    // long (64 bit) is large enough for digit_t.
    // typedef digit_t felm_t[NWORDS_FIELD] => digit_t[2] is felm_t => long[2]
    // typedef felm_t f2elm_t[2] => felm_t[2] is f2elm_t is digit_t[2][2] => long[2][2]
    // point_affine => {f2elm_t x; f2elm_t y;} => {long[2][2] x; long[2][2] y;}
    // typedef point_affine point_t[1] => point_affine[1] is point_t (array with size 1 for passing as reference) => we can ignore that for Java
    //
    // JNA:
    //
    // Variant with pointer (for variant with objects see repo history - much slower):
    // on point needs space for two long[2][2] (2 * 2 * 128 bits = 64 bytes)
    // simply reserve memory (Memory class with 64 bytes) and let c code handle it as the pointer never is used in Java.

    // Fixed-base scalar multiplication Q = k*G, where G is the generator
    // bool ecc_mul_fixed(digit_t* k, point_t Q);
    void ecc_mul_fixed(byte[] privateKey, Pointer p);

    // Encode point P
    // void encode(point_t P, unsigned char* Pencoded)
    void encode(Pointer p, byte[] privateKey);

    // schnorrq with K12 instead of sha-512
    void sign(byte[] subSeed, byte[] publicKey, byte[] messageDigest, byte[] signature);

    // custom schnorrq with precomputed k (careful - only use, if you know what you are doing). Standard: k12 hash
    // sub seed to 64 bytes.
    void signWithNonceK(byte[] nonceK, byte[] publicKey, byte[] messageDigest, byte[] signature);

    // schnorrq with K12 instead of sha-512
    boolean verify(byte[] publicKey, byte[] messageDigest, byte[] signature);

}
