package at.qubic.api.domain.event.response;

import at.qubic.api.util.AssetUtil;
import at.qubic.api.util.BufferUtil;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * Event information for asset ownership and asset possession change.
 */
@Builder
@Getter
@EqualsAndHashCode
@ToString
public class AssetChangeEvent {

    /*
        m256i sourcePublicKey;
        m256i destinationPublicKey;
        m256i issuerPublicKey;
        long long numberOfShares;
        char name[7];
        char numberOfDecimalPlaces;
        char unitOfMeasurement[7];
     */

    private final byte[] sourcePublicKey;
    private final byte[] destinationPublicKey;
    private final byte[] issuerPublicKey;
    private final long numberOfShares;
    private final String name;
    private final byte numberOfDecimalPlaces;
    private final String unitOfMeasurement;

    public static AssetChangeEvent fromBytes(byte[] bytes) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        AssetChangeEvent event = AssetChangeEvent.builder()
                .sourcePublicKey(BufferUtil.getByteArray(buffer, 32))
                .destinationPublicKey(BufferUtil.getByteArray(buffer, 32))
                .issuerPublicKey(BufferUtil.getByteArray(buffer, 32))
                .numberOfShares(buffer.getLong())
                .name(AssetUtil.nameToString(BufferUtil.getByteArray(buffer, 7)))
                .numberOfDecimalPlaces(buffer.get())
                .unitOfMeasurement(AssetUtil.unitOfMeasurementToString(BufferUtil.getByteArray(buffer, 7)))
                .build();
        assert !buffer.hasRemaining();
        return event;
    }

}
