package at.qubic.api.domain.event.response;

import at.qubic.api.util.AssetUtil;
import at.qubic.api.util.BufferUtil;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * Event information for asset issuance events.
 */
@Builder
@Getter
@EqualsAndHashCode
@ToString
public class AssetIssuanceEvent {

    /*
        m256i issuerPublicKey;
        long long numberOfShares;
        char name[7];
        char numberOfDecimalPlaces;
        char unitOfMeasurement[7];
    */

    private final byte[] issuerPublicKey;
    private final long numberOfShares;
    private final String name;
    private final String unitOfMeasurement;
    private final byte numberOfDecimalPlaces;

    public static AssetIssuanceEvent fromBytes(byte[] bytes) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes)
                .order(ByteOrder.LITTLE_ENDIAN);
        AssetIssuanceEvent event = AssetIssuanceEvent.builder()
                .issuerPublicKey(BufferUtil.getByteArray(buffer, 32))
                .numberOfShares(buffer.getLong())
                .name(AssetUtil.nameToString(BufferUtil.getByteArray(buffer, 7)))
                .unitOfMeasurement(AssetUtil.unitOfMeasurementToString(BufferUtil.getByteArray(buffer, 7)))
                .numberOfDecimalPlaces(buffer.get())
                .build();
        assert !buffer.hasRemaining();
        return event;
    }

}
