package at.qubic.api.domain.event.response;

import at.qubic.api.util.BufferUtil;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * Event information for qu transfer events.
 */
@Builder
@Getter
@EqualsAndHashCode
@ToString
public class QuTransferEvent {

    /*
        m256i sourcePublicKey;
        m256i destinationPublicKey;
        long long amount;
     */

    private final byte[] sourcePublicKey;
    private final byte[] destinationPublicKey;
    private final long amount; // unsigned long

    public static QuTransferEvent fromBytes(byte[] input) {
        ByteBuffer buf = ByteBuffer.wrap(input)
                .order(ByteOrder.LITTLE_ENDIAN);
        QuTransferEvent event = QuTransferEvent.builder()
                .sourcePublicKey(BufferUtil.getByteArray(buf, 32))
                .destinationPublicKey(BufferUtil.getByteArray(buf, 32))
                .amount(buf.getLong())
                .build();
        assert buf.remaining() == 0;
        return event;
    }

}
