package at.qubic.api.domain.event.response;

import at.qubic.api.util.AssetUtil;
import at.qubic.api.util.BufferUtil;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;
import lombok.experimental.SuperBuilder;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@SuperBuilder
@Getter
@EqualsAndHashCode(callSuper = true)
@ToString
public class QxTradeMessageEvent extends ContractInformationEvent {

    /*
        struct _TradeMessage
        {
            unsigned int _contractIndex;
            unsigned int _type;

            id issuer;
            uint64 assetName;
            sint64 price;
            sint64 numberOfShares;

            char _terminator;
        } _tradeMessage;
     */

    private final byte[] issuer;
    private final String assetName;
    private final long price;
    private final long numberOfShares;

    public static QxTradeMessageEvent fromBytes(byte[] bytes) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        QxTradeMessageEvent event = QxTradeMessageEvent.builder()
                .contractIndex(buffer.getInt())
                .type(buffer.getInt())
                .issuer(BufferUtil.getByteArray(buffer, 32))
                .assetName(AssetUtil.nameToString(BufferUtil.getByteArray(buffer, 8)))
                .price(buffer.getLong())
                .numberOfShares(buffer.getLong())
                .build();
        assert !buffer.hasRemaining();
        return event;
    }

}
