package at.qubic.api.domain.qearn.response;

import at.qubic.api.domain.qearn.QearnConversionUtil;
import lombok.Builder;
import lombok.Data;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Data
public class BurnedAndBoostedStats {

    private final long burnedAmount;
    private final long averageBurnedPercent;
    private final long boostedAmount;
    private final long averageBoostedPercent;
    private final long rewardedAmount;
    private final long averageRewardedPercent;

    public static BurnedAndBoostedStats fromBytes(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        BurnedAndBoostedStats response = fromBuffer(buffer);
        assert buffer.remaining() == 0;
        return response;
    }

    private static BurnedAndBoostedStats fromBuffer(ByteBuffer buffer) {
        return BurnedAndBoostedStats.builder()
                .burnedAmount(buffer.getLong())
                .averageBurnedPercent(buffer.getLong())
                .boostedAmount(buffer.getLong())
                .averageBoostedPercent(buffer.getLong())
                .rewardedAmount(buffer.getLong())
                .averageRewardedPercent(buffer.getLong())
                .build();
    }

    public BigDecimal getAverageBurnedPercentAdDecimal() {
        return QearnConversionUtil.toDecimalPercentage(averageBurnedPercent);
    }

    public BigDecimal getAverageBoostedPercentAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(averageBoostedPercent);
    }

    public BigDecimal getAverageRewardedPercentAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(averageRewardedPercent);
    }

}
