package at.qubic.api.domain.qearn.response;

import at.qubic.api.domain.qearn.QearnConversionUtil;
import lombok.Builder;
import lombok.Data;
import lombok.Getter;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Data
@Getter
public class BurnedAndBoostedStatsForEpoch {

    private final long burnedAmount;
    private final long burnedPercent;
    private final long boostedAmount;
    private final long boostedPercent;
    private final long rewardedAmount;
    private final long rewardedPercent;

    public static BurnedAndBoostedStatsForEpoch fromBytes(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        BurnedAndBoostedStatsForEpoch response = fromBuffer(buffer);
        assert buffer.remaining() == 0;
        return response;
    }

    private static BurnedAndBoostedStatsForEpoch fromBuffer(ByteBuffer buffer) {
        return BurnedAndBoostedStatsForEpoch.builder()
                .burnedAmount(buffer.getLong())
                .burnedPercent(buffer.getLong())
                .boostedAmount(buffer.getLong())
                .boostedPercent(buffer.getLong())
                .rewardedAmount(buffer.getLong())
                .rewardedPercent(buffer.getLong())
                .build();
    }

    public BigDecimal getBurnedPercentAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(burnedPercent);
    }

    public BigDecimal getBoostedPercentAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(boostedPercent);
    }

    public BigDecimal getRewardedPercentAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(rewardedPercent);
    }

}
