package at.qubic.api.domain.qearn.response;

import at.qubic.api.domain.qearn.QearnConversionUtil;
import lombok.Builder;
import lombok.Data;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Data
public class LockInfoForEpoch {

    private final long lockedAmount;
    private final long bonusAmount;
    private final long currentLockedAmount;
    private final long currentBonusAmount;
    private final long yield;

    public static LockInfoForEpoch fromBytes(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        LockInfoForEpoch response = fromBuffer(buffer);
        assert buffer.remaining() == 0;
        return response;
    }

    private static LockInfoForEpoch fromBuffer(ByteBuffer buffer) {
        return LockInfoForEpoch.builder()
                .lockedAmount(buffer.getLong())
                .bonusAmount(buffer.getLong())
                .currentLockedAmount(buffer.getLong())
                .currentBonusAmount(buffer.getLong())
                .yield(buffer.getLong())
                .build();
    }

    public BigDecimal getYieldPercentage() {
        return QearnConversionUtil.toDecimalPercentage(yield);
    }

}
