package at.qubic.api.domain.qearn.response;

import lombok.Builder;
import lombok.Data;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Data
public class StateOfRound {

    public enum Status {
        OPEN, RUNNING, ENDED;

        private static Status forOrdinal(int code) {
            for (Status status : values()) {
                if (status.ordinal() == code) {
                    return status;
                }
            }
            throw new IllegalArgumentException("Unknown status code " + code);
        }

    }

    private final int state;

    public static StateOfRound fromBytes(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        StateOfRound response = fromBuffer(buffer);
        assert buffer.remaining() == 0;
        return response;
    }

    private static StateOfRound fromBuffer(ByteBuffer buffer) {
        return StateOfRound.builder()
                .state(buffer.getInt())
                .build();
    }

    public Status getStatus() {
        return Status.forOrdinal(state);
    }

}
