package at.qubic.api.domain.qearn.response;

import at.qubic.api.domain.qearn.QearnConversionUtil;
import lombok.Builder;
import lombok.Data;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Data
public class StatsForEpoch {

    // This call behaves strangly. Only part of it is for the current epoch.

    private final long earlyUnlockedAmount; // early unlocked amount in current epoch
    private final long earlyUnlockedPercent; // percentage in relation to total locked amount
    private final long totalLockedAmount; // total amount locked over all epochs
    private final long averageApy; // average apy over all epochs

    public static StatsForEpoch fromBytes(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        StatsForEpoch response = fromBuffer(buffer);
        assert buffer.remaining() == 0;
        return response;
    }

    private static StatsForEpoch fromBuffer(ByteBuffer buffer) {
        return StatsForEpoch.builder()
                .earlyUnlockedAmount(buffer.getLong())
                .earlyUnlockedPercent(buffer.getLong())
                .totalLockedAmount(buffer.getLong())
                .averageApy(buffer.getLong())
                .build();
    }

    public BigDecimal getEarlyUnlockedPercentAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(earlyUnlockedPercent);
    }

    public BigDecimal getAverageApyAsDecimal() {
        return QearnConversionUtil.toDecimalPercentage(averageApy);
    }

}
