package at.qubic.api.domain.qearn.response;

import lombok.Builder;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.List;

@Builder
@Data
public class UserLockStatus {

    /*
        the status is in binary format. For example:
        1101010010110101001011010100101101010010110101001001

        1 means: locked in epoch x weeks ago. 0 means: not locked in epoch x weeks ago.
        Documentation in core code is wrong. Needs to be interpreted from right to left.
        For example 100110 means locked 1, 2, and 5 epochs ago.
     */
    private final long status; // uint64

    public static UserLockStatus fromBytes(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        UserLockStatus response = fromBuffer(buffer);
        assert buffer.remaining() == 0;
        return response;
    }

    private static UserLockStatus fromBuffer(ByteBuffer buffer) {
        return UserLockStatus.builder()
                .status(buffer.getLong())
                .build();
    }

    public String getStatusAsBinaryString() {
        return StringUtils.leftPad(Long.toBinaryString(status), 52, '0');
    }

    public List<Integer> getEpochOffsetsWithLocks() {
        String statusString = StringUtils.reverse(getStatusAsBinaryString());
        List<Integer> epochOffsets = new ArrayList<>();
        char[] statusArray = statusString.toCharArray();
        for (int i = 0; i < statusArray.length; i++) {
            if (statusArray[i] == '1') {
                epochOffsets.add(i);
            }
        }
        return epochOffsets;
    }

}
