package at.qubic.api.domain.qx.request;

import lombok.Builder;
import lombok.Getter;
import lombok.ToString;
import org.apache.commons.codec.binary.Hex;
import at.qubic.api.util.AssetUtil;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@ToString
@Builder
public class QxAssetOrderData {

    public static final short SIZE = (short) 32 + 8 + 8 + 8;

    @ToString.Exclude
    @Getter
    private final byte[] assetIssuer; // unit8_t issuer[32] - public key
    @Getter
    private final String assetName; // 8 bytes - unint64_t assetName;
    @Getter
    private final long price; // long long // price per share
    @Getter
    private final long numberOfShares; // long long

    public byte[] toBytes() {
        return ByteBuffer.allocate(SIZE)
                .order(ByteOrder.LITTLE_ENDIAN)
                .put(assetIssuer)
                .put(AssetUtil.nameTo8Bytes(assetName))
                .putLong(price)
                .putLong(numberOfShares)
                .array();
    }

    @SuppressWarnings("unused") // to string
    @ToString.Include
    private String issuerPublicKey() {
        return assetIssuer == null ? null : Hex.encodeHexString(assetIssuer);
    }

}
