package at.qubic.api.domain.qx.request;

import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import at.qubic.api.domain.qx.Qx;
import at.qubic.api.domain.RequestContractFunction;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Slf4j
public abstract class QxGetEntityOrders extends RequestContractFunction {

    static final short INPUT_SIZE = 40;

    /**
     * issuer public key
     */
    private final byte[] entityPublicKey; // unit8_t issuer[32] - public key

    /**
     * offset is number of skipped order book entries (probably for paging)
     */
    @Getter
    private final long offset; // unint64_t offset;

    public QxGetEntityOrders(Qx.Function function, byte[] entityPublicKey, long offset) {
        super(Qx.CONTRACT_INDEX, function.getCode());
        this.entityPublicKey = entityPublicKey;
        this.offset = offset;
    }

    @Override
    protected byte[] toBytes() {
        return ByteBuffer.allocate(INPUT_SIZE)
                .order(ByteOrder.LITTLE_ENDIAN)
                .put(entityPublicKey)
                .putLong(offset)
                .array();
    }

}
