package at.qubic.api.domain.qx.request;

import at.qubic.api.domain.TransactionExtraData;
import at.qubic.api.util.AssetUtil;
import lombok.Builder;
import lombok.Getter;
import lombok.ToString;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static at.qubic.api.domain.qx.Qx.Procedure.QX_ISSUE_ASSET;

@Builder
@ToString
public class QxIssueAssetOrder implements TransactionExtraData {

    public static final short SIZE = (short) 8 + 8 + 8 + 8;

    @Getter
    private final String name; // 8 bytes - unint64_t name;

    @Getter
    private final int numberOfUnits; // int64_t numberOfUnits

    /**
     * 8 bytes, first 7 are used. every byte represents the number of factors for the corresponding unit of
     * measurement. For example m2 is 0,0,0,0,0,2,0. String representation 0000020 would be byte value + 48
     */
    @Getter
    private final String unitOfMeasurement; // 8 bytes - uint64_t unitOfMeasurement

    @Getter
    private final byte numberOfDecimalPlaces; // char numberOfDecimalPlaces

    @Getter
    @Builder.Default
    private final long amount = 1_000_000_000L;

    @Override
    public short getInputType() {
        return QX_ISSUE_ASSET.getCode();
    }

    @Override
    public byte[] toBytes() {
        return ByteBuffer.allocate(SIZE)
                .order(ByteOrder.LITTLE_ENDIAN)
                .put(AssetUtil.nameTo8Bytes(name))
                .putLong(numberOfUnits)
                .put(AssetUtil.unitOfMeasurementTo8Bytes(unitOfMeasurement))
                .putLong(numberOfDecimalPlaces) // 4 bytes (auto padding in c++)
                .array();
    }

}
