package at.qubic.api.domain.qx.request;

import at.qubic.api.domain.TransactionExtraData;
import at.qubic.api.util.AssetUtil;
import lombok.Builder;
import lombok.Getter;
import lombok.ToString;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static at.qubic.api.domain.qx.Qx.Procedure.QX_TRANSFER_SHARE;

@Builder
@ToString
public class QxTransferAssetOrder implements TransactionExtraData {

    public static final short SIZE = (short) 32 + 32 + 8 + 8;

    @ToString.Exclude
    @Getter
    private final byte[] assetIssuer; // unit8_t issuer[32] - public key

    @ToString.Exclude
    @Getter
    private final byte[] newOwnerAndPossessor; // unit8_t issuer[32] - public key

    @Getter
    private final String assetName; // 8 bytes

    @Getter
    private final long numberOfUnits; // long long

    @Getter
    @Builder.Default
    private final long amount = 100;

    @Override
    public short getInputType() {
        return QX_TRANSFER_SHARE.getCode();
    }

    @Override
    public byte[] toBytes() {
        return ByteBuffer.allocate(SIZE)
                .order(ByteOrder.LITTLE_ENDIAN)
                .put(assetIssuer)
                .put(newOwnerAndPossessor)
                .put(AssetUtil.nameTo8Bytes(assetName))
                .putLong(numberOfUnits)
                .array();
    }

}
