package at.qubic.api.domain.qx.response;

import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.ArrayUtils;
import at.qubic.api.util.BufferUtil;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Slf4j
@Builder
@Getter
@EqualsAndHashCode
public class AssetOrder {

    public static final int ORDER_COUNT = 256;
    public static final int ORDER_SIZE_BYTES = 48; // 32 + 8 + 8

    private final byte[] entity; // uint8_t[32] entity public key
    private final long price; // long long
    private final long numberOfShares; // long long

    // qx call responds with max 256 asset orders
    public static List<AssetOrder> getAssetOrders(byte[] input) {
        ByteBuffer buffer = ByteBuffer.wrap(input);
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        List<AssetOrder> assetOrders = new ArrayList<>(ORDER_COUNT);
        AssetOrder assetOrder = AssetOrder.fromBytes(buffer);
        // skip empty orders at end
        while (!assetOrder.isEmpty() && assetOrders.size() < ORDER_COUNT-1) {
            assetOrders.add(assetOrder);
            assetOrder = AssetOrder.fromBytes(buffer);
        }
        return assetOrders;
    }

    private static AssetOrder fromBytes(ByteBuffer buffer) {
        return AssetOrder.builder()
                .entity(BufferUtil.getByteArray(buffer, 32))
                .price(buffer.getLong())
                .numberOfShares(buffer.getLong())
                .build();
    }

    public boolean isEmpty() {
        return ArrayUtils.isEmpty(entity) || Arrays.equals(entity, new byte[32]);
    }

    @Override
    public String toString() {
        return "AssetOrder(" +
                "entity=" + (entity == null ? null : Hex.encodeHexString(entity)) +
                ", price=" + price +
                ", numberOfShares=" + numberOfShares +
                ')';
    }
}
