package at.qubic.api.domain.qx.response;

import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Getter
@EqualsAndHashCode
@ToString
public class QxFees {

    public static final int SIZE_BYTES = 12;

    private final int assetIssuanceFee; // uint32_t
    private final int transferFee; // uint32_t
    private final int tradeFee; // uint32_t

    public static QxFees fromBytes(byte[] input) {
        if (input.length != SIZE_BYTES) {
            throw new IllegalArgumentException("Incorrect bytes input size.");
        }
        ByteBuffer buf = ByteBuffer.wrap(input);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        return QxFees.builder()
                .assetIssuanceFee(buf.getInt())
                .transferFee(buf.getInt())
                .tradeFee(buf.getInt())
                .build();
    }

    public BigDecimal getTradeFeePercentage() {
        return new BigDecimal(tradeFee).movePointLeft(7).stripTrailingZeros();
    }

}
