package at.qubic.api.domain.std;

import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Setter // for transaction hash
@Getter
@SuperBuilder
public class SignedTransaction {

    private String transactionHash; // don't add to byte representation
    private final Transaction transaction;
    private byte[] signature; // 64 bytes

    protected ByteBuffer toByteBuffer(int capacity) {
        ByteBuffer buf = transaction.toByteBuffer(capacity);
        if (buf.hasRemaining()) {
            buf.put(signature);
        }
        assert buf.remaining() == 0;
        return buf;
    }

    public byte[] toBytes() {
        return toByteBuffer(transaction.getBytesLength() + ArrayUtils.getLength(signature))
                .array();
    }

    public static SignedTransaction fromBytes(byte[] input) {
        ByteBuffer buf = ByteBuffer.wrap(input);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        final Transaction transaction = Transaction.fromByteBuffer(buf, true);
        SignedTransaction signedTransaction = SignedTransaction.builder()
                .transaction(transaction)
                .signature(getSignature(buf))
                .build();
        assert buf.remaining() == 0;
        return signedTransaction;
    }

    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
                .appendSuper(super.toString())
                .append("transactionHash", transactionHash)
                .append("signature", signature == null ? null : Hex.encodeHexString(signature))
                .build();
    }

    private static byte[] getSignature(ByteBuffer buf) {
        byte[] arr = new byte[64];
        buf.get(arr);
        return arr;
    }

}
