package at.qubic.api.domain.std.request;

import lombok.Builder;
import lombok.ToString;
import at.qubic.api.domain.QubicRequest;
import at.qubic.api.domain.MessageType;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@ToString(onlyExplicitlyIncluded = true)
public class RequestTickTransactions extends QubicRequest {

    @ToString.Include
    private final int tick;

    // transactions flags need to be filled with 0 for every 8 transactions to be retrieved. The rest should be filled with 255.
    @ToString.Include
    @Builder.Default
    private final byte[] transactionFlags = new byte[128]; // unsigned char transactionFlags[NUMBER_OF_TRANSACTIONS_PER_TICK / 8]; 1024/8=128

    @Override
    public MessageType getMessageType() {
        return MessageType.REQUEST_TICK_TRANSACTIONS;
    }

    public byte[] getMessagePayload() {
        ByteBuffer buf = ByteBuffer.allocate(132); // 128 + 4
        buf.order(ByteOrder.LITTLE_ENDIAN);
        buf.putInt(tick);
        buf.put(transactionFlags);
        return buf.array();
    }

}
