package at.qubic.api.domain.std.response;

import at.qubic.api.util.AssetUtil;
import at.qubic.api.util.BufferUtil;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import java.nio.ByteBuffer;
import java.util.Arrays;

@Builder
@Getter
@EqualsAndHashCode
public class AssetIssuance {

    private final byte[] publicKey; // char[32] => byte[32]
    private final byte type; // char => byte - share = 1, token = 2 TODO map
    private final byte[] name; // char[7] => byte[7] / capital letters + digits
    private final byte numberOfDecimalPlaces; // char => byte
    private final byte[] unitOfMeasurement; // char[7] => byte[7] / powers of the corresponding SI base units going in alphabetical order

    public static AssetIssuance fromByteBuffer(ByteBuffer buf) {
        return AssetIssuance.builder()
                .publicKey(BufferUtil.getByteArray(buf, 32))
                .type(buf.get())
                .name(BufferUtil.getByteArray(buf, 7))
                .numberOfDecimalPlaces(buf.get())
                .unitOfMeasurement(BufferUtil.getByteArray(buf, 7))
                .build();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
                .append("publicKey", publicKey == null ? null : Hex.encodeHexString(publicKey))
                .append("type", type)
                .append("name", AssetUtil.nameToString(name))
                .append("numberOfDecimalPlaces", numberOfDecimalPlaces)
                .append("unitOfMeasurement", Arrays.toString(unitOfMeasurement))
                .build();
    }

}
