package at.qubic.api.domain.std.response;

import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import at.qubic.api.util.BufferUtil;

import java.nio.ByteBuffer;

@Builder
@Getter
@EqualsAndHashCode
public class AssetPossession {

    private final byte[] publicKey; // char[32] => byte[32]
    private final byte type; // char
    private final byte padding; // char[1] => byte[1] / byte
    private final short managingContractIndex; // unsigned short - 2 bytes
    private final int ownershipIndex; // unsigned int - 4 bytes
    private final long numberOfUnits; // long long - 8 bytes

    public static AssetPossession fromByteBuffer(ByteBuffer buf) {
        return AssetPossession.builder()
                .publicKey(BufferUtil.getByteArray(buf, 32))
                .type(buf.get())
                .padding(buf.get())
                .managingContractIndex(buf.getShort())
                .ownershipIndex(buf.getInt())
                .numberOfUnits(buf.getLong())
                .build();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
                .append("publicKey", publicKey == null ? null : Hex.encodeHexString(publicKey))
                .append("type", type)
                .append("managingContractIndex", managingContractIndex)
                .append("ownershipIndex", ownershipIndex)
                .append("numberOfUnits", numberOfUnits)
                .build();
    }

}
