package at.qubic.api.domain.std.response;

import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

@Builder
@Getter
@EqualsAndHashCode
public class ExchangePublicPeers {

    private final String[] peers;

    public static ExchangePublicPeers fromBytes(byte[] input) {
        ByteBuffer buf = ByteBuffer.wrap(input);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        String[] peers = new String[4];
        for (int i = 0; i < peers.length; i++) {
            peers[i] = String.format("%d.%d.%d.%d", Byte.toUnsignedInt(buf.get()), Byte.toUnsignedInt(buf.get()), Byte.toUnsignedInt(buf.get()), Byte.toUnsignedInt(buf.get()));
        }
        // FIXME assert buf.remaining() == 0 : buf.remaining(); conflicts with toBytes
        return ExchangePublicPeers.builder()
                .peers(peers)
                .build();
    }

    public ByteBuffer toBytes() {
        ByteBuffer buf = ByteBuffer.allocate(64); // TODO check why 64? shouldn't it be 16?
        buf.order(ByteOrder.LITTLE_ENDIAN);
        for (String peer : peers) {
            byte[] ip = ipStringToInts(peer);
            buf.put(ip[0]);
            buf.put(ip[1]);
            buf.put(ip[2]);
            buf.put(ip[3]);
        }
        return buf;
    }

    byte[] ipStringToInts(String ip) {
        final Byte[] numbers = Arrays.stream(StringUtils.split(ip, "."))
                .map(s -> (byte) Integer.parseInt(s)) // narrowing results in unsigned int (discards all but 8 bits)
                .toList().toArray(new Byte[0]);
        return ArrayUtils.toPrimitive(numbers);
    }


}
