package at.qubic.api.domain.std.response;

import lombok.Builder;
import lombok.Getter;
import lombok.ToString;
import at.qubic.api.util.BufferUtil;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Getter
@ToString
public class RespondOwnedAssets {

    private final AssetOwnership asset; // 48 bytes
    private final AssetIssuance issuanceAsset; // 48 bytes
    private final int tick; // unsigned int - 4 bytes
    private final int universeIndex; // unsigned int - 4 bytes
    @Builder.Default @ToString.Exclude
    private final byte[][] siblings = new byte[24][32]; //unsigned char siblings[ASSETS_DEPTH][32]

    public static RespondOwnedAssets fromBytes(byte[] bytes) {
        ByteBuffer buf = ByteBuffer.wrap(bytes);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        AssetOwnership assetOwnership = AssetOwnership.fromByteBuffer(buf);
        AssetIssuance assetIssuance = AssetIssuance.fromByteBuffer(buf);
        int tick = buf.getInt();
        int universeIndex = buf.getInt();
        RespondOwnedAssets roa = RespondOwnedAssets.builder()
                .asset(assetOwnership)
                .issuanceAsset(assetIssuance)
                .tick(tick)
                .universeIndex(universeIndex)
                .siblings(BufferUtil.getByteArrays(buf, 24, 32))
                .build();
        assert buf.remaining() == 0 : buf.remaining();
        return roa;
    }

}
