package at.qubic.api.domain.std.response;

import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.ToString;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

@Builder
@Getter
@EqualsAndHashCode
@ToString
public class TickInfo {

    private final short tickDuration; // unsigned short
    private final short epoch; // unsigned short
    private final int tick; // unsigned int
    private final short numberOfAlignedVotes; // unsigned short
    private final short numberOfMisalignedVotes; // unsigned short
    private final int initialTick; // unsigned int

    public static TickInfo fromBytes(byte[] input) {
        ByteBuffer buf = ByteBuffer.wrap(input);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        TickInfo tickInfo = TickInfo.builder()
                .tickDuration(buf.getShort())
                .epoch(buf.getShort())
                .tick(buf.getInt())
                .numberOfAlignedVotes(buf.getShort())
                .numberOfMisalignedVotes(buf.getShort())
                .initialTick(buf.getInt())
                .build();
        assert buf.remaining() == 0;
        return tickInfo;
    }

    public ByteBuffer toByteBuffer() {
        ByteBuffer buf = ByteBuffer.allocate(16);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        buf.putShort(tickDuration);
        buf.putShort(epoch);
        buf.putInt(tick);
        buf.putShort(numberOfAlignedVotes);
        buf.putShort(numberOfMisalignedVotes);
        buf.putInt(initialTick);
        return buf;
    }

}
