package at.qubic.api.network;

import at.qubic.api.properties.NetworkProperties;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.exception.ExceptionUtils;
import reactor.core.publisher.Flux;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Slf4j
public class FixNodesStrategy extends NodesManagementAdapter {

    public FixNodesStrategy() {
        super(new NoopNetworkStatus());
    }

    public FixNodesStrategy(NetworkStatus networkStatus, NetworkProperties properties) {
        super(networkStatus, properties);
    }

    @Override
    public Flux<Node> getNodes(int max) {
        return super.getNodes(max)
                .switchIfEmpty(Flux.defer(() -> Flux.fromIterable(getRandomNodes(max)))
                        .doOnError(e -> log.error("Error getting random nodes: {}", ExceptionUtils.getMessage(e))));

    }

    private List<Node> getRandomNodes(int max) {
        List<Node> nodeList = new ArrayList<>(nodes.values());
        Collections.shuffle(nodeList);
        return nodeList.stream().limit(max).toList();
    }

}
