package at.qubic.api.network;

import lombok.extern.slf4j.Slf4j;
import at.qubic.api.exception.NoNodeAvailableException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Instant;

@Slf4j
public class Nodes implements NodesManagementStrategy {

    private final NodesManagementStrategy nodesStrategy;

    public Nodes() {
        this.nodesStrategy = new FixNodesStrategy();
    }

    public Nodes(NodesManagementStrategy nodesStrategy) {
        this.nodesStrategy = nodesStrategy;
    }

    public Mono<Node> getNode() {
        return this.getNodes(1)
                .singleOrEmpty()
                .doOnNext(node -> log.debug("Getting node [{}].", node.getName()))
                .switchIfEmpty(Mono.defer(Nodes::noNodeAvailableError));
    }

    private static Mono<Node> noNodeAvailableError() {
        return Mono.error(new NoNodeAvailableException("Could not get node. No node available."));
    }

    @Override
    public Node addNode(String host) {
        return nodesStrategy.addNode(host);
    }

    @Override
    public Flux<Node> getNodes(int max) {
        return nodesStrategy.getNodes(max);
    }

    @Override
    public Mono<Instant> forceNetworkUpdate(Instant updateIfBefore) {
        return nodesStrategy.forceNetworkUpdate(updateIfBefore);
    }

}
