package at.qubic.api.network;

import lombok.extern.slf4j.Slf4j;
import at.qubic.api.properties.NetworkProperties;
import reactor.core.scheduler.Schedulers;

@Slf4j
public class PublicNodesStrategy extends NodesManagementAdapter implements PeerListener {

    public PublicNodesStrategy(NetworkStatus networkStatus) {
        super(networkStatus);
    }

    public PublicNodesStrategy(NetworkStatus networkStatus, NetworkProperties properties) {
        super(networkStatus, properties);
    }

    @Override
    protected Node newNode(String host, int port) {
        Node node = super.newNode(host, port);
        node.addPeerListener(this);
        node.getCurrentTickInfo()
                .subscribeOn(Schedulers.boundedElastic())
                .subscribe(x -> {},
                        // we could open the circuit breaker here so that we ignore this node for some time
                        e -> log.warn("Error getting initial tick info for new node [{}].", host));
        return node;
    }

    @Override
    public void publishPeers(String[] hosts) {
        for (String host : hosts) {
            addNode(host);
        }
    }

}
