package at.qubic.api.service;

import at.qubic.api.domain.TransactionExtraData;
import at.qubic.api.domain.qearn.Qearn;
import at.qubic.api.domain.std.SignedTransaction;
import lombok.extern.slf4j.Slf4j;

import java.util.function.Function;

@Slf4j
public class QearnTransactionService {

    private final TransactionService transactionService;

    public QearnTransactionService(TransactionService transactionService) {
        this.transactionService = transactionService;
    }

    public Function<Integer, SignedTransaction> createTransaction(String seed, TransactionExtraData extraData, int offset) {
        log.info("Creating signed transaction with seed and extra data: {}", extraData);
        return tick -> transactionService.createTransaction(
                tick+ offset,
                seed,
                Qearn.ADDRESS,
                extraData.getAmount(),
                extraData.getInputType(),
                extraData.toBytes());
    }

    public Function<Integer, SignedTransaction> createTransactionWithNonceK(byte[] nonceK, String from, TransactionExtraData extraData, int offset) {
        return tick -> transactionService.createTransactionWithNonceK(
                tick + offset,
                nonceK,
                from,
                Qearn.ADDRESS,
                extraData.getAmount(),
                extraData.getInputType(),
                extraData.toBytes()
        );
    }

}
