package at.qubic.api.service;

import lombok.extern.slf4j.Slf4j;
import at.qubic.api.domain.qx.Qx;
import at.qubic.api.domain.std.SignedTransaction;
import at.qubic.api.domain.TransactionExtraData;

import java.util.function.Function;

@Slf4j
public class QxTransactionService {

    private final TransactionService transactionService;

    public QxTransactionService(TransactionService transactionService) {
        this.transactionService = transactionService;
    }

    public Function<Integer, SignedTransaction> createTransactionFunction(String seed, TransactionExtraData extraData, int offset) {
        log.info("Creating signed transaction with seed and extra data: {}", extraData);
        return tick -> transactionService.createTransaction(
                tick + offset,
                seed,
                Qx.ADDRESS,
                extraData.getAmount(),
                extraData.getInputType(),
                extraData.toBytes());
    }

    public Function<Integer, SignedTransaction> createTransactionFunctionWithNonceK(byte[] nonceK, String from, TransactionExtraData extraData, int offset) {
        log.info("Creating signed transaction function with nondce and extra data: {}", extraData);
        return tick -> transactionService.createTransactionWithNonceK(
                tick + offset,
                nonceK,
                from,
                Qx.ADDRESS,
                extraData.getAmount(),
                extraData.getInputType(),
                extraData.toBytes());
    }

    public SignedTransaction createTransaction(String seed, TransactionExtraData extraData, int tick) {
        log.info("Creating signed transaction with extra data: {}", extraData);
        return transactionService.createTransaction(
                tick,
                seed,
                Qx.ADDRESS,
                extraData.getAmount(),
                extraData.getInputType(),
                extraData.toBytes());
    }

    public SignedTransaction createTransactionWithNonceK(byte[] nonceK, String from, TransactionExtraData extraData, int tick) {
        log.info("Creating signed transaction with nonce and extra data: {}", extraData);
        return transactionService.createTransactionWithNonceK(
                tick,
                nonceK,
                from,
                Qx.ADDRESS,
                extraData.getAmount(),
                extraData.getInputType(),
                extraData.toBytes());
    }

}
