package at.qubic.api.domain;

import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static at.qubic.api.domain.MessageType.UNKNOWN;
import static at.qubic.api.domain.QubicHeader.HEADER_LENGTH;

class QubicHeaderTest {

    @Test
    void setIntSize() {
        assertThat(requestHeaderSize(0)).containsExactly(0, 0, 0);
        assertThat(requestHeaderSize(1)).containsExactly(1, 0, 0);
        assertThat(requestHeaderSize(8)).containsExactly(8, 0, 0);
        assertThat(requestHeaderSize(257)).containsExactly(1, 1, 0);
        assertThat(requestHeaderSize(65535)).containsExactly(-1, -1, 0);
        assertThat(requestHeaderSize(65536)).containsExactly(0, 0, 1);
        // in c++ unsigned int is used (max 65536)
        assertThat(requestHeaderSize(2147483647)).containsExactly(-1, -1, -1);
    }

    @Test
    void fromBytes() {
        final QubicHeader header = QubicHeader.builder()
                .type(UNKNOWN)
                .size(HEADER_LENGTH + 777)
                .build();

        final QubicHeader result = QubicHeader.fromBytes(header.toBytes(HEADER_LENGTH).flip());
        assertThat(result).isEqualTo(header);

    }

    private byte[] requestHeaderSize(int size) {
        return QubicHeader.sizeToBytes(size);
    }

}