package at.qubic.api.domain;

import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.util.Random;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;
import static at.qubic.api.domain.MessageType.UNKNOWN;
import static at.qubic.api.domain.QubicHeader.HEADER_LENGTH;

class QubicMessageTest {

    @Test
    void fromBytes() {
        QubicMessage original = QubicMessage.builder()
                .header(QubicHeader.builder()
                        .type(UNKNOWN)
                        .size(HEADER_LENGTH + 32)
                        .build())
                .payload(randomBytes())
                .build();

        final QubicMessage result = QubicMessage.fromBytes(ByteBuffer.wrap(original.toBytes()));

        assertThat(result).isEqualTo(original);
    }

    @Test
    void fromBytes_notEnoughBytes_thenThrow() {
        final QubicHeader header = QubicHeader.builder()
                .type(UNKNOWN)
                .size(HEADER_LENGTH + 769)
                .build();

        assertThatThrownBy(() -> QubicMessage.fromBytes(header.toBytes(674).put(new byte[666]).flip()))
                .isInstanceOf(IllegalStateException.class);
    }

    private static byte[] randomBytes() {
        byte[] publicKey = new byte[32];
        new Random().nextBytes(publicKey);
        return publicKey;
    }

}