package at.qubic.api.domain.event.response;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.api.crypto.NoCrypto;
import org.bouncycastle.util.encoders.Base64;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;

class AssetChangeEventTest {

    private final IdentityUtil identityUtil = new IdentityUtil(true, new NoCrypto());

    @Test
    void fromBytes() {
        byte[] input = Base64.decode("0VGbNisG/WoCJ31lNTqScnrtWbF+ZuwLYqLQJkopJv02EYTUnxm1rKM15TYeDdxsn6lv0WHZd47t7Tzvs+MeIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAQAAAAAAAABNTE0AAAAAAAAAAAAAAAA=");
        AssetChangeEvent event = AssetChangeEvent.fromBytes(input);
        assertThat(event).isNotNull();
        assertThat(event).isEqualTo(AssetChangeEvent.builder()
                        .sourcePublicKey(identityUtil.getPublicKeyFromIdentity("BOJOBRHAZILUCDADNGBXYUIYHNIDCKBSQEWGUFCAJASIOPFNBMWWXDJHCCTC"))
                        .destinationPublicKey(identityUtil.getPublicKeyFromIdentity("UBGIZFTLNCRKAFJBIUUYWTOMEFEDVPTZZZBKNSZODERUXBJPWQZSPGYAMCYH"))
                        .issuerPublicKey(identityUtil.getPublicKeyFromIdentity("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAFXIB"))
                        .numberOfShares(1)
                        .name("MLM")
                        .unitOfMeasurement("0000000")
                        .numberOfDecimalPlaces((byte) 0)
                .build());

    }

    @Test
    void fromBytes_givenCfbAsset_thenSetUnitOfMeasurement() {
        byte[] input = Base64.decode("sMmo18V9WMO9LstUtxvWC2ZfJc2/FZWKEUdAKOqNKDIvyKKaekppac06VyRMSMUCe1tpQO0R9znQUrQOndNX+ggwu2O/fV4WSsjL04aAYw/3Zwoevzn3IQtAvNyiU9Bf2XE+AAAAAABDRkIAAAAAAADQANAjGBU=");
        AssetChangeEvent event = AssetChangeEvent.fromBytes(input);
        assertThat(event).isNotNull();
        assertThat(event.getName()).isEqualTo("CFB");
        assertThat(event.getUnitOfMeasurement()).isEqualTo("0\0000\000SHE"); // QWALLET and CFB were issued with wrong unit of measurement: A^(0)*cd^(-48)*K^(0)*kg^(-48)*m^(35)*mol^(24)*s^(21)
    }

}