package at.qubic.api.domain.qearn.request;

import at.qubic.api.domain.qearn.Qearn;
import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.assertThat;

class GetStateOfRoundTest {

    @Test
    void getMessagePayload() {
        GetStateOfRound request = GetStateOfRound.builder()
                .epoch(123)
                .build();

        byte[] payload = request.getMessagePayload();
        assertThat(payload).hasSize(12);

        ByteBuffer buffer = ByteBuffer.wrap(payload).order(ByteOrder.LITTLE_ENDIAN);
        assertThat(buffer.getInt()).isEqualTo(Qearn.CONTRACT_INDEX);
        assertThat(buffer.getShort()).isEqualTo(Qearn.Function.QEARN_GET_STATE_OF_ROUND.getCode());
        assertThat(buffer.getShort()).isEqualTo(GetStateOfRound.INPUT_SIZE);
        assertThat(buffer.remaining()).isEqualTo(GetStateOfRound.INPUT_SIZE); // see toBytes()
    }

    @Test
    void toBytes() {
        GetStateOfRound request = GetStateOfRound.builder()
                .epoch(123)
                .build();

        byte[] bytes = request.toBytes();
        assertThat(bytes).hasSize(GetStateOfRound.INPUT_SIZE);

        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        assertThat(buffer.getInt()).isEqualTo(123);
    }

}