package at.qubic.api.domain.qearn.request;

import at.qubic.api.domain.qearn.Qearn;
import at.qubic.api.util.BufferUtil;
import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.*;

class GetUserLockInfoTest {

    private static final byte[] PUB_KEY = {1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2};

    @Test
    void getMessagePayload() {
        byte[] bytes = testRequest().getMessagePayload();

        assertThat(bytes).hasSize(44);
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        assertThat(buffer.getInt()).isEqualTo(Qearn.CONTRACT_INDEX);
        assertThat(buffer.getShort()).isEqualTo(Qearn.Function.QEARN_GET_USER_LOCKED_INFO.getCode());
        assertThat(buffer.getShort()).isEqualTo(GetUserLockInfo.INPUT_SIZE);
        assertThat(buffer.remaining()).isEqualTo(GetUserLockInfo.INPUT_SIZE); // see toBytes()
    }

    @Test
    void toBytes() {
        byte[] bytes = testRequest().toBytes();

        assertThat(bytes).hasSize(GetUserLockInfo.INPUT_SIZE);
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        assertThat(BufferUtil.getByteArray(buffer, 32)).isEqualTo(PUB_KEY);
        assertThat(buffer.getInt()).isEqualTo(123);
    }

    private static GetUserLockInfo testRequest() {
        return GetUserLockInfo.builder()
                .publicKey(PUB_KEY)
                .epoch(123)
                .build();
    }

}