package at.qubic.api.domain.qearn.request;

import at.qubic.api.domain.qearn.Qearn;
import at.qubic.api.util.BufferUtil;
import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.assertThat;

class GetUserLockStatusTest {

    private static final byte[] PUB_KEY = {1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2};

    @Test
    void getMessagePayload() {
        byte[] bytes = GetUserLockStatus.builder()
                .publicKey(PUB_KEY)
                .build()
                .getMessagePayload();

        assertThat(bytes).hasSize(40);
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        assertThat(buffer.getInt()).isEqualTo(Qearn.CONTRACT_INDEX);
        assertThat(buffer.getShort()).isEqualTo(Qearn.Function.QEARN_GET_USER_LOCKED_STATE.getCode());
        assertThat(buffer.getShort()).isEqualTo(GetUserLockStatus.INPUT_SIZE);
        assertThat(buffer.remaining()).isEqualTo(GetUserLockStatus.INPUT_SIZE); // see toBytes()
    }

    @Test
    void toBytes() {
        byte[] bytes = GetUserLockStatus.builder()
                .publicKey(PUB_KEY)
                .build()
                .toBytes();

        assertThat(bytes).hasSize(GetUserLockStatus.INPUT_SIZE);
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
        assertThat(BufferUtil.getByteArray(buffer, 32)).isEqualTo(PUB_KEY);
    }

}