package at.qubic.api.domain.qearn.response;

import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.*;

class BurnedAndBoostedStatsPerEpochTest {

    @Test
    void fromBytes() {
        byte[] bytes = ByteBuffer.allocate(6 * 8)
                .order(ByteOrder.LITTLE_ENDIAN)
                .putLong(1)
                .putLong(2)
                .putLong(3)
                .putLong(4)
                .putLong(5)
                .putLong(6)
                .array();

        BurnedAndBoostedStatsForEpoch status = BurnedAndBoostedStatsForEpoch.fromBytes(bytes);
        assertThat(status).isNotNull();
        assertThat(status.getBurnedAmount()).isEqualTo(1);
        assertThat(status.getBurnedPercent()).isEqualTo(2);
        assertThat(status.getBoostedAmount()).isEqualTo(3);
        assertThat(status.getBoostedPercent()).isEqualTo(4);
        assertThat(status.getRewardedAmount()).isEqualTo(5);
        assertThat(status.getRewardedPercent()).isEqualTo(6);
    }

    @Test
    void getAverageBurnedPercentAsDecimal() {
        BurnedAndBoostedStatsForEpoch status = BurnedAndBoostedStatsForEpoch.builder()
                .burnedPercent(1)
                .boostedPercent(2)
                .rewardedPercent(3)
                .build();
        assertThat(status.getBurnedPercentAsDecimal()).isEqualTo("0.00001");
        assertThat(status.getBoostedPercentAsDecimal()).isEqualTo("0.00002");
        assertThat(status.getRewardedPercentAsDecimal()).isEqualTo("0.00003");
    }

}