package at.qubic.api.domain.qearn.response;

import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.assertThat;

class BurnedAndBoostedStatsTest {

    @Test
    void fromBytes() {
        byte[] bytes = ByteBuffer.allocate(6 * 8)
                .order(ByteOrder.LITTLE_ENDIAN)
                .putLong(1)
                .putLong(2)
                .putLong(3)
                .putLong(4)
                .putLong(5)
                .putLong(6)
                .array();

        BurnedAndBoostedStats status = BurnedAndBoostedStats.fromBytes(bytes);
        assertThat(status).isNotNull();
        assertThat(status.getBurnedAmount()).isEqualTo(1);
        assertThat(status.getAverageBurnedPercent()).isEqualTo(2);
        assertThat(status.getBoostedAmount()).isEqualTo(3);
        assertThat(status.getAverageBoostedPercent()).isEqualTo(4);
        assertThat(status.getRewardedAmount()).isEqualTo(5);
        assertThat(status.getAverageRewardedPercent()).isEqualTo(6);
    }

    @Test
    void getAverageBurnedPercentAsDecimal() {
        BurnedAndBoostedStats status = BurnedAndBoostedStats.builder()
                .averageBurnedPercent(1)
                .averageBoostedPercent(2)
                .averageRewardedPercent(3)
                .build();
        assertThat(status.getAverageBurnedPercentAdDecimal()).isEqualTo("0.00001");
        assertThat(status.getAverageBoostedPercentAsDecimal()).isEqualTo("0.00002");
        assertThat(status.getAverageRewardedPercentAsDecimal()).isEqualTo("0.00003");
    }

}