package at.qubic.api.domain.qearn.response;

import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.*;

class StatsForEpochTest {

    @Test
    void fromBytes() {
        byte[] bytes = ByteBuffer.allocate(32)
                .order(ByteOrder.LITTLE_ENDIAN)
                .putLong(1)
                .putLong(2)
                .putLong(3)
                .putLong(4)
                .array();

        StatsForEpoch lockInfo = StatsForEpoch.fromBytes(bytes);
        assertThat(lockInfo.getEarlyUnlockedAmount()).isEqualTo(1);
        assertThat(lockInfo.getEarlyUnlockedPercent()).isEqualTo(2);
        assertThat(lockInfo.getTotalLockedAmount()).isEqualTo(3);
        assertThat(lockInfo.getAverageApy()).isEqualTo(4);
    }

    @Test
    void getPercentages() {
        StatsForEpoch lockInfo = StatsForEpoch.builder()
                .earlyUnlockedPercent(1)
                .averageApy(2)
                .build();

        assertThat(lockInfo.getEarlyUnlockedPercentAsDecimal()).isEqualTo("0.00001");
        assertThat(lockInfo.getAverageApyAsDecimal()).isEqualTo("0.00002");
    }

}