package at.qubic.api.domain.qearn.response;

import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.List;

import static org.assertj.core.api.Assertions.*;

class UserLockStatusTest {

    @Test
    void fromBytes() {
        byte[] bytes = ByteBuffer.allocate(8)
                .order(ByteOrder.LITTLE_ENDIAN)
                .putLong(123)
                .array();

        UserLockStatus userLockStatus = UserLockStatus.fromBytes(bytes);
        assertThat(userLockStatus.getStatus()).isEqualTo(123);
    }

    @Test
    void getStatusAsBinaryString() {
        String lockStatus = "11001";
        UserLockStatus userLockStatus = UserLockStatus.builder()
                .status(Long.parseLong(lockStatus, 2))
                .build();
        // zero padded to 52 characters
        assertThat(userLockStatus.getStatusAsBinaryString()).isEqualTo("00000000000000000000000000000000000000000000000" + lockStatus);
    }

    @Test
    void getEpochOffsetsWithLocks() {
        String lockStatus = "1001000000000000000001100000001101001010110100101011";
        UserLockStatus userLockStatus = UserLockStatus.builder()
                .status(Long.parseLong(lockStatus, 2))
                .build();
        List<Integer> lockedEpochs = userLockStatus.getEpochOffsetsWithLocks();
        assertThat(lockedEpochs).containsExactly(0, 1, 3, 5, 8, 10, 11, 13, 15, 18, 20, 21, 29, 30, 48, 51);
    }
}