package at.qubic.api.domain.qx.request;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.assertThat;

@Slf4j
class QxAssetOrderDataTest {

    @Test
    void toBytes() {
        QxAssetOrderData orderData = QxAssetOrderData.builder()
                .assetIssuer(new byte[32])
                .assetName("FOO")
                .price(123)
                .numberOfShares(45)
                .build();

        log.info("Order data: {}", orderData);

        byte[] bytes = orderData.toBytes();
        assertThat(bytes).isNotNull();
        assertThat(bytes).hasSize(QxAssetOrderData.SIZE);
        assertThat(bytes).isEqualTo(ByteBuffer.allocate(QxAssetOrderData.SIZE)
                .order(ByteOrder.LITTLE_ENDIAN)
                .put(new byte[32])
                .put(new byte[] {'F', 'O', 'O', 0,0,0,0,0})
                .putLong(123)
                .putLong(45)
                .array());
        assertThat(Hex.encodeHexString(bytes)).isEqualTo(
                "0000000000000000000000000000000000000000000000000000000000000000" +
                        "464f4f0000000000" +
                        "7b00000000000000" +
                        "2d00000000000000");
    }

}