package at.qubic.api.domain.qx.request;

import at.qubic.api.domain.qx.Qx;
import at.qubic.api.util.AssetUtil;
import org.junit.jupiter.api.Test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.*;

class QxGetAssetAskOrdersTest {

    @Test
    void getMessagePayload() {
        byte[] expectedIssuer = new byte[32];
        expectedIssuer[31] = 9;
        QxGetAssetAskOrders getOrders = new QxGetAssetAskOrders(expectedIssuer, "TEST", 123);
        byte[] payload = getOrders.getMessagePayload();
        assertThat(payload).hasSize(56);

        ByteBuffer buffer = ByteBuffer.wrap(payload).order(ByteOrder.LITTLE_ENDIAN);

        assertThat(buffer.getInt()).isEqualTo(Qx.CONTRACT_INDEX);
        assertThat(buffer.getShort()).isEqualTo(Qx.Function.QX_GET_ASSET_ASK_ORDER.getCode());
        assertThat(buffer.getShort()).isEqualTo(QxGetAssetOrders.INPUT_SIZE);

        byte[] assetIssuer = new byte[32];
        buffer.get(assetIssuer);
        assertThat(assetIssuer).isEqualTo(expectedIssuer);

        byte[] assetName = new byte[8];
        buffer.get(assetName);
        assertThat(assetName).isEqualTo(AssetUtil.nameTo8Bytes("TEST"));

        assertThat(buffer.getLong()).isEqualTo(123);
    }

}