package at.qubic.api.domain.std.response;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;
import org.junit.jupiter.api.Test;
import at.qubic.api.crypto.IdentityUtil;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.assertThat;

class AssetIssuanceTest {

    private final IdentityUtil identityUtil = new IdentityUtil();
    private static final String assetHex = "000000000000000000000000000000000000000000000000000000000000000001515800000000000000000000000000";

    @Test
    void fromByteBuffer() throws DecoderException {
        ByteBuffer buffer = ByteBuffer.wrap(Hex.decodeHex(assetHex));
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        AssetIssuance asset = AssetIssuance.fromByteBuffer(buffer);
        assertThat(asset.getType()).isEqualTo((byte) 1);
        assertThat(identityUtil.getIdentityFromPublicKey(asset.getPublicKey())).isEqualTo("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAFXIB");
        assertThat(Hex.encodeHexString(asset.getPublicKey())).isEqualTo("0000000000000000000000000000000000000000000000000000000000000000");
        assertThat(new String(asset.getName()).trim()).isEqualTo("QX");
        assertThat(asset.getNumberOfDecimalPlaces()).isEqualTo((byte) 0);
        assertThat(asset.getUnitOfMeasurement()).isEqualTo(new byte[7]);
    }

}