package at.qubic.api.domain.std.response;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;
import org.junit.jupiter.api.Test;
import at.qubic.api.crypto.IdentityUtil;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import static org.assertj.core.api.Assertions.*;

class AssetPossessionTest {

    private final IdentityUtil identityUtil = new IdentityUtil();
    private static final String assetHex = "bba07fde537ea438760811910f2d874f017974eadcfdde70aa771533aa94064c030001004477ab003200000000000000";

    @Test
    void fromByteBuffer() throws DecoderException {

        ByteBuffer buffer = ByteBuffer.wrap(Hex.decodeHex(assetHex));
        buffer.order(ByteOrder.LITTLE_ENDIAN);
        AssetPossession asset = AssetPossession.fromByteBuffer(buffer);

        assertThat(asset).isNotNull();
        assertThat(identityUtil.getIdentityFromPublicKey(asset.getPublicKey())).isEqualTo("TESTIOYEZSAUQBUNKJEGXOILIBICZYWAMVSWYCYFHDCSZOAXPTGSOKFCCUNK");
        assertThat(Hex.encodeHexString(asset.getPublicKey())).isEqualTo("bba07fde537ea438760811910f2d874f017974eadcfdde70aa771533aa94064c");
        assertThat(asset.getType()).isEqualTo((byte) 3);
        assertThat(asset.getManagingContractIndex()).isEqualTo((short) 1);
        assertThat(asset.getOwnershipIndex()).isEqualTo(11237188);
        assertThat(asset.getNumberOfUnits()).isEqualTo((short) 50);
    }

}