package at.qubic.api.network;

import at.qubic.api.domain.QubicRequest;
import at.qubic.api.SpringBootTestConfiguration;
import at.qubic.api.crypto.IdentityUtil;
import at.qubic.api.domain.MessageType;
import at.qubic.api.domain.std.request.BroadcastTransaction;
import at.qubic.api.domain.std.request.RequestEntity;
import at.qubic.api.service.TransactionService;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

@Slf4j
@Tag("SIT")
@SpringBootTest(classes = SpringBootTestConfiguration.class, properties = { "use.public.peers=false", "check.health=true" })
class BasicNodeOperationsSIT {

    @Autowired
    private Nodes nodes;

    @Autowired
    private TransactionService transactionService;

    @Autowired
    private IdentityUtil identityUtil;

    private final BasicNodeOperations nodeOperations = new BasicNodeOperations();

    @Test
    void sendAndReceive() {
        final Mono<Node> node = nodes.getNode();

        final Mono<QubicRequest> requestMessage = Mono.just("MMPYLHEBCADPHFOXPTTSBRWJRNSBHEVFXCXEHRBIXFDTQCLJRFRKZGGFFQUM")
                .doOnNext(id -> log.info("Query id [{}]", id))
                .map(identityUtil::getPublicKeyFromIdentity)
                .map(pub -> RequestEntity.builder().publicKey(pub).build());

        StepVerifier.create(node.flatMap(n -> nodeOperations.sendAndReceiveNext(n, requestMessage, MessageType.RESPOND_ENTITY)
                        .doOnNext(m -> log.info(m.getHeader().getType().toString()))))
                .expectNextCount(1)
                .verifyComplete();
    }

    @Test
    void send() {
        StepVerifier.create(nodes.getNode().flatMap(n -> n.getCurrentTickInfo()
                .map(ti -> transactionService.createTransaction(ti.getTick() + 5,
                        "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                        "BZBQFLLBNCXEMGLOBHUVFTLUPLVCPQUASSILFABOFFBCADQSSUPNWLZBQEXK",
                        1
                ))
                .map(signedTransaction -> BroadcastTransaction.builder().transaction(signedTransaction).build())
                .flatMap(request -> nodeOperations.send(n, Mono.just(request)))))
                .expectNextCount(1)
                .verifyComplete();

    }

}