package at.qubic.api.network;

import org.junit.jupiter.api.Test;
import at.qubic.api.exception.NoNodeAvailableException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import java.time.Instant;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class NodesTest {

    private final NodesManagementStrategy strat = mock();
    private final Nodes nodes = new Nodes(strat);

    @Test
    void getNode() {
        Node expected = mock();
        when(strat.getNodes(1)).thenReturn(Flux.fromIterable(List.of(expected)));
        StepVerifier.create(nodes.getNode())
                .expectNext(expected)
                .verifyComplete();
    }

    @Test
    void getNode_givenMultiple_thenThrow() {
        when(strat.getNodes(1)).thenReturn(Flux.fromIterable(List.of(mock(), mock())));
        StepVerifier.create(nodes.getNode())
                .expectError(IndexOutOfBoundsException.class)
                .verify();
    }

    @Test
    void getNode_givenNone_thenThrow() {
        when(strat.getNodes(1)).thenReturn(Flux.empty());
        StepVerifier.create(nodes.getNode())
                .expectError(NoNodeAvailableException.class)
                .verify();
    }

    @Test
    void getNode_getDefaultFromFixedStrategy() {
        Nodes nodes = new Nodes();
        Node node = nodes.addNode("localhost");
        StepVerifier.create(nodes.getNode())
                .expectNext(node)
                .verifyComplete();
    }

    @Test
    void forceNetworkUpdate() {
        when(strat.forceNetworkUpdate(Instant.MIN)).thenReturn(Mono.just(Instant.MAX));
        StepVerifier.create(nodes.forceNetworkUpdate(Instant.MIN))
                .expectNext(Instant.MAX)
                .verifyComplete();

    }

}