package at.qubic.api.network;

import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;
import at.qubic.api.domain.std.response.TickInfo;

import java.time.Instant;

import static org.assertj.core.api.Assertions.assertThat;

class TickStatusTest {

    @ParameterizedTest
    @CsvSource(textBlock = """
            0,0,0,false
            1,0,0,false
            0,1,0,false
            0,0,1,false
            1,1,0,false
            1,0,1,false
            0,1,1,false
            1,1,1,true
            6,7,8,true
            """)
    void isValid(short epoch, int tick, long epochSecond, boolean expected) {

        Instant instant = Instant.ofEpochSecond(epochSecond);

        TickStatus ts = TickStatus.builder()
                .tickInfo(TickInfo.builder()
                        .epoch(epoch)
                        .tick(tick)
                        .build())
                .latestUpdate(instant)
                .build();

        assertThat(ts.isValid()).isEqualTo(expected);

    }

    @ParameterizedTest
    @CsvSource(textBlock = """
            0,0,false
            0,1,true
            1,0,false
            12345,12346,true
            12345,12344,false
            """)
    void lastUpdatedBefore(long updatedEpochSeconds, long testEpochSeconds, boolean expected) {
        Instant updatedAt = Instant.ofEpochSecond(updatedEpochSeconds);
        Instant validAt = Instant.ofEpochSecond(testEpochSeconds);

        TickStatus ts = TickStatus.builder()
                .tickInfo(TickInfo.builder()
                        .build())
                .latestUpdate(updatedAt)
                .build();

        assertThat(ts.lastUpdatedBefore(validAt)).isEqualTo(expected);
    }

}