package at.qubic.api.service;

import org.junit.jupiter.api.Test;
import at.qubic.api.domain.TransactionExtraData;
import at.qubic.api.domain.qx.Qx;
import at.qubic.api.domain.std.SignedTransaction;
import at.qubic.api.domain.std.response.TickInfo;

import java.util.function.Function;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class QxTransactionServiceTest {

    private final TransactionService transactionService = mock();
    private final QxTransactionService qxTransactionService = new QxTransactionService(transactionService);

    @Test
    void createTransactionFunction() {
        TransactionExtraData txExtraData = mock();
        when(txExtraData.getAmount()).thenReturn(123L);
        when(txExtraData.getInputType()).thenReturn((short) 45);
        byte[] extraData = {1, 2, 3, 4, 5, 6, 7, 8, 9};
        when(txExtraData.toBytes()).thenReturn(extraData);

        TickInfo tickInfo = TickInfo.builder().tick(656).build();

        SignedTransaction expected = SignedTransaction.builder().build();
        when(transactionService.createTransaction(666, "seed", Qx.ADDRESS, 123L, (short) 45, extraData))
                .thenReturn(expected);

        Function<Integer, SignedTransaction> fx = qxTransactionService.createTransactionFunction("seed", txExtraData, 10);
        SignedTransaction stx = fx.apply(tickInfo.getTick());

        assertThat(stx).isEqualTo(expected);
    }
}