package at.qubic.api.tcp;

import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import at.qubic.api.SpringBootTestConfiguration;
import at.qubic.api.domain.std.response.TickData;
import at.qubic.api.service.ComputorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import static org.assertj.core.api.Assertions.assertThat;

// enable tcp logger for analysis, if one of these tests fails
@Tag("SIT")
@SpringBootTest(classes = SpringBootTestConfiguration.class)
class QubicHeaderFrameDecoderSIT {

    @Autowired
    private ComputorService computorService;

    @Test
    void queryBalance_multipleTimes_thenProcessOneMessagePerByteBuffer() {

        // typically one TCP packet contains one qubic message but sometimes one packet contains multiple messages
        // combined into one packet or one message is split over multiple packets. We need to handle all these cases.
        // For query balance we most of the time get two packets - one with 24B (exchange public peers) and one with 848B (respond entity) -
        // but sometimes only one with 872B containing both messages. enable configured tcp client logger for more details.

        // update: this scenario should be handled by QubicHeaderFrameDecoder now. But the test case is still valid.

        for (int i = 0; i < 10; i++) {
            StepVerifier.create(computorService.getEntity("TESTJFDGICVGFHLRAZCKJKQYVEUBVYWTMRYYTPFXRESOQJGTKHCIHDJGFQZJ"))
                    .expectNextCount(1)
                    .verifyComplete();
        }

    }

    @Test
    void getLatestTickData_combineSeveralPackagesIntoOneFrame() {

        // tick data is split over multiple tcp packets
        // processing tick data will only work if the tcp client combines them into a single byte buffer

        final Mono<TickData> getTickData = computorService.getCurrentTickInfo()
                .flatMap(ti -> computorService.getTickData(ti.getTick() - 1));

        StepVerifier.create(getTickData)
                .assertNext(tickData -> assertThat(tickData.getSignature()).isNotEmpty())
                .verifyComplete();

    }


}