package at.qubic.api.util;

import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;

class AssetUtilTest {

    @Test
    void nameToString() {
        assertThat(AssetUtil.nameToString(new byte[] {'F', 'O', 'O', 0, 0, 0, 0, 0})).isEqualTo("FOO");
    }

    @Test
    void nameTo8Bytes() {
        assertThat(AssetUtil.nameTo8Bytes("FOO")).isEqualTo(new byte[] {'F', 'O', 'O', 0, 0, 0, 0, 0});
        assertThat(AssetUtil.nameTo8Bytes("FOO" + "FOO")).isEqualTo(new byte[] {'F', 'O', 'O', 'F', 'O', 'O', 0, 0});
        assertThat(AssetUtil.nameTo8Bytes("FOO" + "FOO" + "FOO")).isEqualTo(new byte[] {'F', 'O', 'O', 'F', 'O', 'O', 'F', 0});
    }

    @Test
    void unitOfMeasurementTo8Bytes() {
        assertThat(AssetUtil.unitOfMeasurementTo8Bytes("01234560")).isEqualTo(new byte[] {0,1,2,3,4,5,6,0});
        assertThat(AssetUtil.unitOfMeasurementTo8Bytes("0123456789")).isEqualTo(new byte[] {0,1,2,3,4,5,6,0});
        assertThat(AssetUtil.unitOfMeasurementTo8Bytes("0123456")).isEqualTo(new byte[] {0,1,2,3,4,5,6,0});
        assertThat(AssetUtil.unitOfMeasurementTo8Bytes("01002")).isEqualTo(new byte[] {0,1,0,0,2,0,0,0});
        assertThat(AssetUtil.unitOfMeasurementTo8Bytes(null)).isEqualTo(new byte[] {0,0,0,0,0,0,0,0});
    }

    @Test
    void unitOfMeasurementToString() {
        assertThat(AssetUtil.unitOfMeasurementToString(new byte[] {0,1,2,3,4,5,6,0})).isEqualTo("0123456");
        assertThat(AssetUtil.unitOfMeasurementToString(new byte[] {0,0,0,0,0,0,0,1})).isEqualTo("0000000");
        assertThat(AssetUtil.unitOfMeasurementToString(new byte[] {0,1,2,0,0,0,2,0})).isEqualTo("0120002");
        assertThat(AssetUtil.unitOfMeasurementToString(new byte[] {0,1,2,3,4,5,6,7})).isEqualTo("0123456");
    }
}