<img alt="Qubicj Shell Logo" src="src/main/resources/qubicj-shell-logo.png" title="Qubicj Shell Logo" style="float: right"/>

# qubic shell (qubicj-shell)

A shell that provides tools and wallet functionality on the command line. While it should run on windows it is mainly
tested on Linux and some advanced features in non-interactive mode might be restricted to bash.

In development. Currently supported features:

* Manage several wallets.
* Send qubics (transfers).
* Query entity information.
* Query owned and possessed assets.
* Query network information.
* Full QX support.
* Full QEARN support.
* Read only wallets.
* Auto-completion.
* Node pooling.

I am happy about feedback. Please write me on discord or open an issue on gitlab.

## Installation

* Install Java 21
* **IMPORTANT** Install shared qubic crypto lib (see instructions in [qubicj-computor-api](../qubicj-computor-api/README.md)).
* Install the application executable (download from artifacts section in gitlab or build it yourself).
* Add custom configuration

### Setup example for Linux System 

If you use bash (if you don't you know it and don't need this section anyway) you can do the following for easier setup:

* Create a directory where the executable is created, for example `~/app/qubicj/`.
* Put the executable inside this directory and create a symlink to it.
* Create an alias for starting the application and add it to your shell config file like `.bashrc` or `.bash_aliases`.

For example (make sure to replace x.y.z and to understand the commands first):

```shell
 mkdir -p ~/app/qubicj # create directory
 cd ~/app/qubicj # change into the directory
 ln -s qubicj-shell.jar /path/to/executable/qubicj-shell-x.y.z.jar # create symbolic link for easier access to the file
 alias qubicjshell='java -jar $HOME/app/qubicj-shell/qubicj-shell.jar' # add alias for appliation
```

You can now run the shell with `qubicjshell` from the command line. To make this change persistent add the following to
`.bashrc` or `.bash_aliases` if you use such a file. For example:
```
echo 'alias qubicjshell="java -jar $HOME/app/qubicj-shell/qubicj-shell.tar"' >> .bash_aliases
```

## Build

Follow the [build instructions in the parent project](../README.md#build).
After building the executable can be found in the submodules `target/` directory.

## Usage

### Configuration

You can override any configurable settings by creating a properties file in your application directory (recommended).
By default, the file is named `qubicj.properties` and located in `${user.home}/.qubicj`.
This directory also holds other configuration and wallet information.

Example configuration file specifying seed nodes

```properties
# disable other profiles first
spring.profiles.active=
# add custom config
computor.hosts=145.239.161.41, 154.51.14.151, 154.51.14.152, 162.19.112.138, 185.130.227.189, 185.132.201.163, 82.197.173.129
network.port=21841
network.refresh-interval=60s
use-public-peers=true
```

#### Advanced configuration

For configuration options see [application.properties](./src/main/resources/application.properties). You can also override properties in
the [default spring way](https://docs.spring.io/spring-boot/reference/features/external-config.html) without using `qubicj.properties`.

### Start in interactive mode

Interactive mode. Start, then type `help` for commands. Example:

```shell
java -jar qubic-shell-<VERSION>.jar
```

### Start in script mode

Using script/batch mode (requires script file containing the commands).

```shell
java -jar qubic-shell-<VERSION>.jar script --file do-something.txt
```

### List of operations

Use `help` command to find out what commands are supported. Use `help <command-name>` to get more details about single
commands.

### Quickstart TL;DR

* Add some wallets.
* Query entities and send transactions.

## Usage examples

This section shows some examples on how to use the available commands. The `help` command always shows the most
accurrate information. The documentation here might get out of date.

### Help

#### General help

```
shell:>help
AVAILABLE COMMANDS

Built-In Commands
       help: Display help about available commands
       stacktrace: Display the full stacktrace of the last error.
       clear: Clear the shell screen.
       quit, exit: Exit the shell.
       history: Display or save the history of previously run commands
       version: Show version info
       script: Read and execute commands from a file.

entity
       balance: Retrieve entity balance information.
       assets owned: Retrieve owned assets.
       assets possessed: Retrieve possessed assets.
       send: Transfer qubics from one identity to another.

keys
       password: Set a new password.
       unlock: Unlock key store.
       verify: Checks the integrity of the stored keys.
       lock: Lock key store.

network
       network tick-info: Show current epoch and tick information.

qearn
       qearn ended-status: Show qearn info about lock periods that ended this or previous epoch.
       qearn info: Show qearn information. You can request overall information or details per epoch and/or identity.
       qearn unlock: Unlock previously locked amount from qearn.
       qearn lock: Lock amount in qearn in current epoch.

qx
       qx entity-orders: Shows orders of an entity.
       qx remove-ask: Removes ask (sell) order for an asset.
       qx add-bid: Adds bid (buy) order for an asset.
       qx remove-bid: Removes bid (buy) order for an asset.
       qx add-ask: Adds ask (sell) order for an asset.
       qx transfer-asset: Sends assets from one entity to another.
       qx fees: Show qx fees.
       qx order-book: Shows orders for an asset.

wallet
       wallet add-seed: Add the seed to a wallet or create a new wallet.
       wallet remove: Delete a wallet.
       wallet add: Add a (read only) wallet.
       wallet list: List wallet identities.
       wallet remove-seed: Remove the seed from a wallet.
```

#### Detailed help

Every command offers detailed help where all options and default values are listed. Use the `--help` or `-h` option.

For example get detailed help for the `qx add-bid` command:

```       
shell:>qx add-bid --help
NAME
       qx add-bid - Adds bid (buy) order for an asset.

SYNOPSIS
       qx add-bid [--from String] [--asset-name String] --issuer String [--price-per-share long] [--number-of-shares long] --offset int --retries int --validate boolean --help 

OPTIONS
       --from String
       Identity (source).
       [Mandatory]

       --asset-name String
       Name of the asset
       [Mandatory]

       --issuer String
       Issuer of the asset.
       [Optional]

       --price-per-share long
       Limit price to pay per share.
       [Mandatory]

       --number-of-shares long
       Number of desired shares.
       [Mandatory]

       --offset int
       Tick offset.
       [Optional, default = 5]

       --retries int
       Number of retries.
       [Optional, default = 0]

       --validate boolean
       Validate transaction before sending.
       [Optional, default = true]

       --help or -h 
       help for qx add-bid
       [Optional]
```

### Entity Commands

#### Query entity balance

Query the `balance` of an identity:

```
shell:>balance --identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL
```

#### Query owned assets

Shows assets that are owned by the entity.

```
shell:>assets owned --identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL
```

#### Query possessed assets

Shows assets that are in possession of the entity.

```
shell:>assets possessed --identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL 
```

#### Send qubics (make a transaction)

You need a wallet with private key information for sending qus, and you need to unlock the wallet first (see below).

```
shell:>send --amount 1 --from TESTMOFGUMGEAAGPTUKGTCIOHIVADPQXOPRFTHOTOBFHWWOOJBCWWULGQYFD --to TESTGGFPUBLZLAFQFBKMJRCBWBHBHTCYHRCLIWEKXEXKTWQKOHEBUOCDHCXC
```

### Network comamnds

Show current and initial tick of epoch

```
shell:>network tick-info 
```

### Wallet commands

#### Add a read only wallet

Read only wallets only need the identity and no keys. You can create as many read only wallets you like,
you don't need to own the keys for them.

```
shell:>wallet add --identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL
```

The `--identity <....>` option provides autocompletion for all identities that are stored as wallets.

#### Add a seed to a wallet

Adds a new wallet or updates an existing wallet. Stores the private key information to the wallet.
These wallets with private key information allow certain operations, like sending transactions.

```
shell:>wallet add-seed 

Attention! Please store your seed in a safe place. The seed cannot be recovered
from the application. First create a secure and random seed and then copy it.

The seed consists of 55 alphabetical lower case letters.

? Enter seed: *******************************************************
Added/updated seed for identity [TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB].
```

For this you need to unlock the key store first (`wallet unlock`). Be aware of the security implications:
as your private key is stored locally (albeit encrypted) there is a risk that it could be stolen by an
attacker. Use the software at your own risk.

#### List wallets

```
shell:>wallet list
```

#### Unlock wallets

The key store allows to store private keys derived from the `seed`.

```
shell:>wallet unlock 
? Enter password ************
Unlocked wallet.
```

When the key store gets unlocked for the first time you can set an initial password. Make sure
to use a safe password as this protects your private keys.

#### Update the keystore password

Changes the current password. Make sure to use a safe password as this protects
your private keys.

```
shell:>wallet password 
? Enter new password ********
? Repeat the password ********
Password successfully updated.
```

The key store needs to be unlocked for updating the password.

#### Lock wallets

This saves the keystore and clears all sensitive information from memory.

```
shell:>wallet lock
Locked wallet.
```

#### Delete a wallet

Removes the wallet and deletes private keys, if stored.

```
shell:>wallet remove --identity TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB 
```

### QX commands

Interact with the QX smart contract (DEX). Assets can be referenced by name and issuer. All the currently available
assets are preconfigured to use the correct issuer but if you want to be perfectly safe to avoid scam assets it is
recommended to manually specify the issuer because there can be assets with the same name but different issuers.

#### Show fees

Shows fees for using the QX smart contract.

```
shell:>qx fees 
```

#### Show order book

Shows order book of an asset. You can use the predefined assets (see auto-completion).

```
shell:>qx order-book --asset-name CFB
```

#### Show orders of an entity

Shows the open orders of one entity (ID).

```
shell:>qx entity-orders --identity SKSJGFUUHOKEMEHENJKPRDCCIOMAKGGMFVRWRNJEYAKTJWWGIWPDSFLHKQAF
```

#### Add bid order

Add a bid order to buy an asset. If the bid price is equal or higher to the lowest ask price in the order book
the order will get executed immediately. Otherwise, it will be added in the order book.

```
shell:>qx add-bid --from MMPYLHEBCADPHFOXPTTSBRWJRNSBHEVFXCXEHRBIXFDTQCLJRFRKZGGFFQUM --asset-name RANDOM --number-of-shares 1 --price-per-share 1500000000
```

#### Add ask order

Add an ask order to sell an asset. If the ask price is equal or lower to the highest bid price in the order book
the order will get executed. Otherwise, it will be added in the order book.

```
shell:>qx add-ask --from GNRDTDTIEAAATAUPJZEQEEZHXPFAOQRAWIRJWGLFDANVSMQAWURDTHHGXGHC --asset-name RANDOM --number-of-shares 2 --price-per-share 1900000000
```

#### Remove bid order

Remove a bid (buy) order you placed before. You need to specify the exact price per share of the order you want to
remove so that it can be identified.

```
shell:>qx remove-bid --from MMPYLHEBCADPHFOXPTTSBRWJRNSBHEVFXCXEHRBIXFDTQCLJRFRKZGGFFQUM --asset-name RANDOM --number-of-shares 1 --price-per-share 1500000000
```

#### Remove ask order

Remove an ask (sell) order you placed before. You need to specify the exact price per share of the order you want to
remove so that it can be identified.

```
shell:>qx remove-ask --from GNRDTDTIEAAATAUPJZEQEEZHXPFAOQRAWIRJWGLFDANVSMQAWURDTHHGXGHC --asset-name RANDOM --number-of-shares 1 --price-per-share 1900000000
```

### QEARN commands

#### Show qearn info

You can show the general info of one epoch or one identity or one identity in one epoch. Examples:

```
shell:>qearn info --epoch 149
...
shell:>qearn info --identity TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB
...
shell:>qearn info --identity TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB --epoch 140
```

#### Lock amount

The followin command would lock 1bn in the current epoch.

``` 
shell:>qearn lock --identity TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB --amount 1000000000
```

#### Unlock amount

For unlocking you need to specify the epoch to unlock from. Use qearn info for an identity to find the epochs when you locked.

``` 
shell:>qearn unlock --identity TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB --amount 1000000000 --epoch 123
```

#### Check ended status

If unlocked recently (current or previous epoch) you can check the unlocked amounts for an identity.

``` 
shell:>qearn ended-status --identity TESTZXJTNEYEYDDPQGNWHRXHBZRDKJXLXXEZEFEXOCQYMEXADCMEUAQFRDTB
```

#### Show info

### Complete Usage Example

```
$ qubicjshell 
              _     _       _       _          _ _
   __ _ _   _| |__ (_) ___ (_)  ___| |__   ___| | |
  / _` | | | | '_ \| |/ __|| | / __| '_ \ / _ \ | |
 | (_| | |_| | |_) | | (__ | | \__ \ | | |  __/ | |
  \__, |\__,_|_.__/|_|\___|/ | |___/_| |_|\___|_|_|
     |_|                 |__/

  qubicj-shell 1.0.0-SNAPSHOT

shell:>help
AVAILABLE COMMANDS

Built-In Commands
       help: Display help about available commands
       stacktrace: Display the full stacktrace of the last error.
       clear: Clear the shell screen.
       quit, exit: Exit the shell.
       history: Display or save the history of previously run commands
       version: Show version info
       script: Read and execute commands from a file.

entity
       balance: Retrieve entity balance information.
       assets owned: Retrieve owned assets.
       assets possessed: Retrieve possessed assets.
       send: Transfer qubics from one identity to another.

keys
       password: Set a new password.
       unlock: Unlock key store.
       verify: Checks the integrity of the stored keys.
       lock: Lock key store.

network
       network tick-info: Show current epoch and tick information.

qearn
       qearn ended-status: Show qearn info about lock periods that ended this or previous epoch.
       qearn info: Show qearn information. You can request overall information or details per epoch and/or identity.
       qearn unlock: Unlock previously locked amount from qearn.
       qearn lock: Lock amount in qearn in current epoch.

qx
       qx entity-orders: Shows orders of an entity.
       qx remove-ask: Removes ask (sell) order for an asset.
       qx add-bid: Adds bid (buy) order for an asset.
       qx remove-bid: Removes bid (buy) order for an asset.
       qx add-ask: Adds ask (sell) order for an asset.
       qx transfer-asset: Sends assets from one entity to another.
       qx fees: Show qx fees.
       qx order-book: Shows orders for an asset.

wallet
       wallet add-seed: Add the seed to a wallet or create a new wallet.
       wallet remove: Delete a wallet.
       wallet add: Add a (read only) wallet.
       wallet list: List wallet identities.
       wallet remove-seed: Remove the seed from a wallet.

shell:>balance --identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL

Identity: CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL
Balance: 12'642'300'221
Incoming amount: 28'220'300'221
Outgoing amount: 15'578'000'000
Incoming transfers: 5207
Outgoing transfers: 4864
Latest incoming tick: 14996737
Latest outgoing tick: 14996756
Tick: 15031493

shell:>assets owned --identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL
Owned assets of identity CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL:

Issuer: CFBMEMZOIDEXQAUXYYSZIURADQLAPWPMNJXQSNVQZAHYVOPYUKKJBJUCTVJL
Name: CFB
Units: 137'227'691'827
Managing contract index: 1
Issuance index: 12267528
Tick: 15031494

shell:>quit
```

## Back up

To back up your data simply save the contents from the `${user.home}/.qubicj` directory including the hidden files
(the log directory and history file is not needed).