package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import at.qubic.shell.component.PasswordComponent;
import at.qubic.shell.command.service.KeyService;
import org.springframework.shell.command.annotation.Command;

@Command(command = "wallet", group = "wallet")
@Slf4j
public class AddSeedCommand {

    private final KeyService keyManagementService;
    private final PasswordComponent passwordComponent;

    public AddSeedCommand(KeyService walletService, PasswordComponent passwordComponent) {
        this.keyManagementService = walletService;
        this.passwordComponent = passwordComponent;
    }

    @Command(command = "add-seed", description = "Add the seed to a wallet or create a new wallet.")
    public String addSeed() {
        if (keyManagementService.isLocked()) {
            return "Wallet needs to be unlocked.";
        } else {
            String warning = """
            
            Attention! Please store your seed in a safe place. The seed cannot be recovered
            from the application. First create a secure and random seed and then copy it.
            
            The seed consists of 55 alphabetical lower case letters.
            
            """;
            String seed = passwordComponent.enterPassword(warning, "Enter seed:");
            if (keyManagementService.isValidSeed(seed)) {
                String identity = keyManagementService.addSeed(seed);
                return String.format("Added/updated seed for identity [%s].", identity);
            } else {
                return "Invalid seed.";
            }
        }
    }

}
