package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import at.qubic.shell.command.service.EntityCommandService;
import org.springframework.shell.command.annotation.Command;
import org.springframework.shell.command.annotation.Option;
import org.springframework.shell.command.annotation.OptionValues;

import static at.qubic.shell.command.SharedCommandConstants.OPTION_DESCRIPTION_IDENTITY;
import static at.qubic.shell.command.SharedCommandConstants.OPTION_VALUE_PROVIDER_IDENTITY;
import static org.springframework.shell.command.CommandRegistration.OptionArity.EXACTLY_ONE;

@Command(group = "entity")
@Slf4j
public class EntityCommands {

    private final EntityCommandService entityCommandService;

    public EntityCommands(EntityCommandService entityCommandService) {
        this.entityCommandService = entityCommandService;
    }

    @Command(command = "balance", description = "Retrieve entity balance information.")
    public String getEntity(
            @OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
            @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE, required = true)
            String identity) {
        return entityCommandService.getEntity(identity);
    }

    @Command(command = "assets owned", description = "Retrieve owned assets.")
    public String getOwnedAssets(
            @OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
            @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE, required = true)
            String identity) {
        return entityCommandService.getOwnedAssets(identity);
    }

    @Command(command = "assets possessed", description = "Retrieve possessed assets.")
    public String getPossessedAssets(
                                  @OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
                                  @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE, required = true)
                                  String identity) {
        return entityCommandService.getPossessedAssets(identity);
    }

}
