package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import at.qubic.shell.command.service.WalletService;
import org.springframework.shell.command.annotation.Command;
import org.springframework.shell.command.annotation.Option;

import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

@Command(command = "wallet", group = "wallet")
@Slf4j
public class ListWalletsCommand {

    public static final String OPTION_DESCRIPTION_FILTER = "Regular expression to filter wallet identities.";

    private final WalletService walletService;

    public ListWalletsCommand(WalletService walletService) {
        this.walletService = walletService;
    }

    @Command(command = "list", description = "List wallet identities.")
    public List<String> listWallets(
            @Option(description = OPTION_DESCRIPTION_FILTER)
            String filter
    ) {
        List<String> identities = walletService.getAllWalletIdentities();
        if (StringUtils.isBlank(filter)) {
            return identities;
        } else {
            try {
                Pattern pattern = Pattern.compile(filter);
                return identities.stream().filter(id -> pattern.matcher(id).matches()).toList();
            } catch (PatternSyntaxException pse) {
                log.warn("Invalid filter pattern: {}.", pse.getMessage());
                return List.of();
            }
        }
    }

}
