package at.qubic.shell.command;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.command.service.QearnService;
import jakarta.validation.constraints.NotBlank;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.shell.command.annotation.Command;
import org.springframework.shell.command.annotation.Option;
import org.springframework.shell.command.annotation.OptionValues;

import static at.qubic.shell.command.SharedCommandConstants.OPTION_DESCRIPTION_IDENTITY;
import static at.qubic.shell.command.SharedCommandConstants.OPTION_VALUE_PROVIDER_IDENTITY;
import static org.springframework.shell.command.CommandRegistration.OptionArity.EXACTLY_ONE;

// TODO test
@Slf4j
@Command(group = "qearn")
public class QearnFunctionCommands {

    private final IdentityUtil identityUtil;
    private final QearnService qearnService;

    public QearnFunctionCommands(IdentityUtil identityUtil, QearnService qearnService) {
        this.identityUtil = identityUtil;
        this.qearnService = qearnService;
    }

    @Command(command = "qearn info", description = "Show qearn information. " +
            "You can request overall information or details per epoch and/or identity.")
    public String getLockInfo(
            @OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
            @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE)
            String identity,
            @Option(description = "Epoch number.", arity = EXACTLY_ONE, defaultValue = "-1")
            int epoch) {

        if (StringUtils.isNotBlank(identity)) {
            if (!identityUtil.isValidIdentity(identity)) {
                return invalidEntityMessage(identity);
            } else if (epoch > 0) { // epoch specified
                return qearnService.getUserLockInfo(identity, epoch);
            } else {
                return qearnService.getUserLockInfo(identity);
            }
        } else if (epoch > 0) {
            return qearnService.getEpochLockInfo(epoch);
        } else {
            return qearnService.getOverallStats();
        }

    }

    @Command(command = "qearn ended-status", description = "Show qearn info about lock periods that ended this or previous epoch.")
    public String getLockInfo(
            @OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
            @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE, required = true)
            @NotBlank String identity) {

            if (!identityUtil.isValidIdentity(identity)) {
                return invalidEntityMessage(identity);
            } else{ // epoch specified
                return qearnService.getEndedStatus(identity);
            }
    }

    private static String invalidEntityMessage(String identity) {
        return String.format("Identity [%s] is invalid.", identity);
    }


}
