package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import at.qubic.shell.command.service.KeyService;
import org.springframework.shell.command.annotation.Command;
import org.springframework.shell.command.annotation.Option;
import org.springframework.shell.command.annotation.OptionValues;

import static at.qubic.shell.command.SharedCommandConstants.OPTION_DESCRIPTION_IDENTITY;
import static at.qubic.shell.command.SharedCommandConstants.OPTION_VALUE_PROVIDER_IDENTITY;
import static org.springframework.shell.command.CommandRegistration.OptionArity.EXACTLY_ONE;

@Command(command = "wallet", group = "wallet")
@Slf4j
public class RemoveSeedCommand {

    private final KeyService keyManagementService;

    public RemoveSeedCommand(KeyService keyManagementService) {
        this.keyManagementService = keyManagementService;
    }

    @Command(command = "remove-seed", description = "Remove the seed from a wallet.")
    public String removeSeed(@OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
                                 @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE, required = true)
                                 String identity) {
         if (keyManagementService.isValidIdentity(identity)) {
            if (keyManagementService.isLocked()) {
                return "Wallet needs to be unlocked.";
            } else if(keyManagementService.containsKeyForIdentity(identity)) {
                keyManagementService.deletePrivateKey(identity);
                return String.format("Deleted private keys for identity [%s].", identity);
            } else {
                return String.format("No private keys stored for identity [%s].", identity);
            }
        } else {
            return String.format("Identity [%s] is invalid. Cannot remove wallet.", identity);
        }
    }

}
