package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import at.qubic.shell.command.service.KeyService;
import at.qubic.shell.command.service.WalletService;
import org.springframework.shell.command.annotation.Command;
import org.springframework.shell.command.annotation.Option;
import org.springframework.shell.command.annotation.OptionValues;

import static at.qubic.shell.command.SharedCommandConstants.OPTION_DESCRIPTION_IDENTITY;
import static at.qubic.shell.command.SharedCommandConstants.OPTION_VALUE_PROVIDER_IDENTITY;
import static org.springframework.shell.command.CommandRegistration.OptionArity.EXACTLY_ONE;

@Command(command = "wallet", group = "wallet")
@Slf4j
public class RemoveWalletCommand {

    private final WalletService walletService;
    private final KeyService keyManagementService;

    public RemoveWalletCommand(WalletService walletService, KeyService keyManagementService) {
        this.walletService = walletService;
        this.keyManagementService = keyManagementService;
    }

    @Command(command = "remove", description = "Delete a wallet.")
    public String  removeWallet(
            @OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
            @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE, required = true)
            String identity) {

        if (walletService.isValidIdentity(identity)) {
            String response = walletService.removeWallet(identity);
            if (keyManagementService.isLocked()) {
                log.info("Keystore is locked. Cannot delete key for identity [{}].", identity);
            } else if (keyManagementService.containsKeyForIdentity(identity)) {
                keyManagementService.deletePrivateKey(identity);
            } // else nothing to delete
            return response;
        } else {
            return String.format("Identity [%s] is invalid. Cannot remove wallet.", identity);
        }

    }

}
