package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import at.qubic.shell.component.PasswordComponent;
import at.qubic.shell.command.service.KeyService;
import org.springframework.shell.command.annotation.Command;

@Command(group = "keys")
@Slf4j
public class UnlockKeyStoreCommand {

    private final KeyService keyManagementService;
    private final PasswordComponent passwordComponent;

    public UnlockKeyStoreCommand(KeyService walletService, PasswordComponent passwordComponent) {
        this.keyManagementService = walletService;
        this.passwordComponent = passwordComponent;
    }

    @Command(command = "unlock", description = "Unlock key store.")
    public String unlock() {
        if (keyManagementService.isLocked()) {
            String password = passwordComponent.enterPassword("Enter password");
            if (passwordComponent.isValidPassword(password)) {
                return keyManagementService.tryToUnlockKeyStore(password);
            } else {
                return "Cannot unlock wallet. Invalid password.";
            }
        } else {
            return "Wallet is already unlocked.";
        }
    }

}
