package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import at.qubic.shell.component.PasswordComponent;
import at.qubic.shell.command.service.KeyService;
import org.springframework.shell.command.annotation.Command;

@Command(group = "keys")
@Slf4j
public class UpdatePasswordCommand {

    private final KeyService keyManagementService;
    private final PasswordComponent passwordComponent;

    public UpdatePasswordCommand(KeyService walletService, PasswordComponent passwordComponent) {
        this.keyManagementService = walletService;
        this.passwordComponent = passwordComponent;
    }

    @Command(command = "password", description = "Set a new password.")
    public String updatePassword() {
        if (keyManagementService.isLocked()) {
            return "Wallet needs to be unlocked.";
        } else {
            String newPassword = passwordComponent.enterPassword("Enter new password");
            String repeated = passwordComponent.enterPassword("Repeat the password");
            if (!StringUtils.equals(newPassword, repeated)) {
                return "Cannot update password. Passwords do not match.";
            } if (passwordComponent.isValidPassword(newPassword)) {
                return keyManagementService.tryToUpdatePassword(newPassword);
            } else {
                return "Cannot update password. Invalid password.";
            }
        }
    }

}
