package at.qubic.shell.command;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import at.qubic.shell.command.service.KeyService;
import at.qubic.shell.wallet.Wallets;
import org.springframework.shell.command.annotation.Command;
import org.springframework.shell.command.annotation.Option;
import org.springframework.shell.command.annotation.OptionValues;

import java.util.List;
import java.util.StringJoiner;

import static at.qubic.shell.command.SharedCommandConstants.OPTION_DESCRIPTION_IDENTITY;
import static at.qubic.shell.command.SharedCommandConstants.OPTION_VALUE_PROVIDER_IDENTITY;
import static org.springframework.shell.command.CommandRegistration.OptionArity.EXACTLY_ONE;

@Command(group = "keys")
@Slf4j
public class VerifyKeysCommand {

    private final KeyService keyManagementService;
    private final Wallets wallets;

    public VerifyKeysCommand(KeyService walletService, Wallets wallets) {
        this.keyManagementService = walletService;
        this.wallets = wallets;
    }

    @Command(command = "verify", description = "Checks the integrity of the stored keys.")
    public String verify(@OptionValues(provider = OPTION_VALUE_PROVIDER_IDENTITY)
                                 @Option(description = OPTION_DESCRIPTION_IDENTITY, arity = EXACTLY_ONE)
                                 String identity) {
        if (keyManagementService.isLocked()) {
            return "Wallet needs to be unlocked.";
        } else if (StringUtils.isBlank(identity)) {
            StringJoiner resultString = new StringJoiner(",\n", "Stored keys for identities:\n", ".");
            List<String> identities = wallets.getWalletIds().stream()
                    .filter(keyManagementService::containsKeyForIdentity)
                    .toList();
            identities.forEach(
                    id -> resultString.add(String.format("[%s]: [%s]", id, keyManagementService.verifyIdentity(id) ? "valid" : "invalid"))
            );
            return resultString.toString();
        } else if (keyManagementService.containsKeyForIdentity(identity)) {
            boolean verified = keyManagementService.verifyIdentity(identity);
            return String.format("Stored key for identity [%s] is [%s].", identity, verified ? "valid" : "invalid");
        } else {
            return String.format("There is no key for identity [%s].", identity);
        }

    }

}
