package at.qubic.shell.command.service;

import lombok.extern.slf4j.Slf4j;
import at.qubic.api.domain.qx.response.AssetOrder;
import at.qubic.api.domain.qx.response.EntityOrder;
import at.qubic.api.domain.qx.response.QxFees;
import at.qubic.api.service.ComputorService;
import reactor.core.publisher.Mono;

import java.util.List;

@Slf4j
public class QxService {

    private static final int NUM_RETRIES = 3;
    private final ComputorService computorService;

    public QxService(ComputorService computorService) {
        this.computorService = computorService;
    }

    public Mono<QxFees> getQxFees() {
        return computorService.getQxFees()
                .retry(NUM_RETRIES);
    }

    public Mono<List<EntityOrder>> getEntityAskOrders(String identity, int offset) {
        return computorService.getQxAskEntityOrders(identity, offset)
                .retry(NUM_RETRIES);
    }

    public Mono<List<EntityOrder>> getEntityBidOrders(String identity, int offset) {
        return computorService.getQxBidEntityOrders(identity, offset)
                .retry(NUM_RETRIES);
    }

    public Mono<List<AssetOrder>> getAssetAskOrders(String issuer, String name, int offset) {
        return computorService.getQxAskAssetOrders(issuer, name, offset)
                .retry(NUM_RETRIES);
    }

    public Mono<List<AssetOrder>> getAssetBidOrders(String issuer, String name, int offset) {
        return computorService.getQxBidAssetOrders(issuer, name, offset)
                .retry(NUM_RETRIES);
    }

}
