package at.qubic.shell.command.service;

import lombok.extern.slf4j.Slf4j;
import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.wallet.Wallet;
import at.qubic.shell.wallet.Wallets;

import java.util.List;

@Slf4j
public class WalletService {

    private final Wallets wallets;
    private final IdentityUtil identityUtil;

    public WalletService(Wallets wallets, IdentityUtil identityUtil) {
        this.wallets = wallets;
        this.identityUtil = identityUtil;
    }

    public List<String> getAllWalletIdentities() {
        return wallets.getWalletIds();
    }

    public String addWallet(String identity) {
        final String response;
        if (isValidIdentity(identity)) {
            if (wallets.containsWalletIdentity(identity)) {
                response = String.format("There is already a wallet registered for identity [%s].", identity);
            } else {
                wallets.putWallet(new Wallet(identity));
                response = String.format("Added wallet with identity [%s].", identity);
            }
        } else {
            response =  String.format("Identity [%s] is invalid. Cannot add wallet.", identity);
        }
        log.info(response);
        return response;
    }

    public String removeWallet(String identity) {
        Wallet wallet = wallets.removeWallet(identity);
        if (wallet == null) {
            return String.format("There is no wallet registered with identity [%s]. Cannot remove.", identity);
        } else {
            return String.format("Removed wallet with id [%s].", wallet.identity());
        }
    }

    public boolean isValidIdentity(String identity) {
        return identityUtil.isValidIdentity(identity);
    }

}
