package at.qubic.shell.component;

import at.qubic.shell.command.service.InclusionCheck;
import at.qubic.shell.component.terminal.TransactionTerminal;
import at.qubic.shell.exception.AfterSendingException;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

@Slf4j
public class InclusionCheckComponent {

    private final InclusionCheck inclusionCheck;

    public InclusionCheckComponent(InclusionCheck inclusionCheck) {
        this.inclusionCheck = inclusionCheck;
    }

    public Mono<InclusionCheck.InclusionStatus> waitForInclusion(TransactionTerminal<?> terminal, int startTick, int targetTick, String transactionHash) {

        return terminal.writeTransactionProgress(startTick, targetTick, startTick)
                .then(inclusionCheck.checkForInclusion(targetTick, transactionHash)
                        .concatMap(status -> terminal.writeTransactionProgress(startTick, targetTick, status.getCheckedTick()).map(x -> status))
                        .skipUntil(status -> status.getSignedTransaction() != null)
                        .next()
                        .doOnError(e -> log.error(e.getMessage(), e))
                        .switchIfEmpty(Mono.defer(() -> error(targetTick, transactionHash)))); // because of last async write
    }

    private static Mono<InclusionCheck.InclusionStatus> error(int targetTick, String transactionHash) {
        String message = String.format(
                "Unexpected empty termination when checking for inclusion of transaction [%s] in tick [%s].",
                transactionHash, targetTick);
        log.error(message);
        return Mono.error(new AfterSendingException(message));
    }

}
