package at.qubic.shell.component;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jline.terminal.Terminal;
import org.springframework.shell.component.StringInput;
import org.springframework.shell.standard.AbstractShellComponent;

@Slf4j
public class PasswordComponent extends AbstractShellComponent {

    public String enterPassword(String prompt) {
        return getMaskedInput(prompt, getTerminal());
    }

    public String enterPassword(String warning, String prompt) {
        Terminal terminal = getTerminal();
        terminal.writer().write(warning);
        return getMaskedInput(prompt, terminal);
    }

    private String getMaskedInput(String prompt, Terminal terminal) {
        StringInput component = new StringInput(terminal, prompt, null);
        component.setResourceLoader(getResourceLoader());
        component.setTemplateExecutor(getTemplateExecutor());
        component.setMaskCharacter('*');
        StringInput.StringInputContext context = component.run(StringInput.StringInputContext.empty());
        return context.getResultValue();
    }

    public boolean isValidPassword(String password) {
        return StringUtils.isNotBlank(password);
    }

}
