package at.qubic.shell.component.terminal;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.component.TransactionContext;
import lombok.extern.slf4j.Slf4j;
import org.jline.terminal.Terminal;

import java.util.function.Supplier;

@Slf4j
public class QearnLockTerminalOutput extends AbstractTransactionTerminal<TransactionContext> {

    private final String identity;
    private final long amount;

    public QearnLockTerminalOutput(IdentityUtil identityUtil, Supplier<Terminal> terminalSupplier, String identity, long amount) {
        super(identityUtil, new TerminalWriter(terminalSupplier));
        this.identity = identity;
        this.amount = amount;
    }

    @Override
    public void writeOperationDescription() {
        terminalWriter.writeAsync(String.format("Locking %s qu in qearn for %s.\n",
                formatUtil.formatUnsigned(amount),
                identity));
    }

    @Override
    public void writeSuccessMessage(TransactionContext tc) {

        // TODO it would be better to use the lock info in the transaction context for verification


        long incomingAmount = tc.getSourceAfter().getIncomingAmount() - tc.getSourceBefore().getIncomingAmount();
        long outgoingAmount = tc.getSourceAfter().getOutgoingAmount() - tc.getSourceBefore().getOutgoingAmount();
        long balanceChange = incomingAmount - outgoingAmount;
        String transactionInformation = getTransactionInformationString(tc.getSignedTransaction());

        String balanceInformation = """
                    Balance change: %s
                    Outgoing amount: %s
                    Latest outgoing tick: %d""".formatted(
                formatUtil.format(balanceChange),
                formatUtil.formatUnsigned(outgoingAmount),
                tc.getSourceAfter().getLatestOutgoingTransferTick());

        // in case we send an amount that does not get locked we get it back as incoming amount
        if (Math.negateExact(balanceChange) >= amount && outgoingAmount >= amount) {
            log.info("Transaction [{}] successful.", tc.getSignedTransaction().getTransactionHash());

            String output = """
               Transaction included. Successfully locked.
              
               %s
               
               %s
               """.formatted(transactionInformation,
                    balanceInformation);

            terminalWriter.writeAsync(output);

        } else { // smaller or most probably no amount unlocked

            log.info("Transaction [{}] included but unsuccessful.", tc.getSignedTransaction().getTransactionHash());
            String output = """
            Transaction included in tick but NOT successful. Could not lock requested amount.
            
            %s
            
            %s
            """.formatted(transactionInformation, balanceInformation);

            terminalWriter.writeAsync(output);

        }

    }

}
