package at.qubic.shell.component.terminal;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.component.TransactionContext;
import lombok.extern.slf4j.Slf4j;
import org.jline.terminal.Terminal;

import java.util.function.Supplier;

@Slf4j
public class QearnUnlockTerminalOutput extends AbstractTransactionTerminal<TransactionContext> {

    private final String identity;
    private final int epoch;
    private final long amount;

    public QearnUnlockTerminalOutput(IdentityUtil identityUtil, Supplier<Terminal> terminalSupplier, String identity, int epoch, long amount) {
        super(identityUtil, new TerminalWriter(terminalSupplier));
        this.identity = identity;
        this.epoch = epoch;
        this.amount = amount;
    }

    @Override
    public void writeOperationDescription() {
        terminalWriter.writeAsync(String.format("Unlocking %s qu from qearn epoch %d for %s.\n",
                formatUtil.formatUnsigned(amount),
                epoch,
                identity));
    }

    @Override
    public void writeSuccessMessage(TransactionContext tc) {

        // TODO it would be better to use the lock info in the transaction context for verification

        long incomingAmount = tc.getSourceAfter().getIncomingAmount() - tc.getSourceBefore().getIncomingAmount();
        long outgoingAmount = tc.getSourceAfter().getOutgoingAmount() - tc.getSourceBefore().getOutgoingAmount();
        long balanceChange = incomingAmount - outgoingAmount;
        String transactionInformation = getTransactionInformationString(tc.getSignedTransaction());

        String balanceInformation = """
                    Balance change: %s
                    Incoming amount: %s
                    Latest incoming tick: %d""".formatted(
                formatUtil.format(balanceChange),
                formatUtil.formatUnsigned(incomingAmount),
                tc.getSourceAfter().getLatestIncomingTransferTick());

        if (incomingAmount >= amount) {
            log.info("Transaction [{}] successful.", tc.getSignedTransaction().getTransactionHash());

            String output = """
               Transaction included. Successfully unlocked.
              
               %s
               
               %s
               """.formatted(transactionInformation,
                    balanceInformation);

            terminalWriter.writeAsync(output);

        } else { // smaller or most probably no amount unlocked

            log.info("Transaction [{}] included but unsuccessful.", tc.getSignedTransaction().getTransactionHash());
            String output = """
            Transaction included in tick but NOT successful. Could not unlock requested amount.
            
            %s
            
            %s
            """.formatted(transactionInformation, balanceInformation);

            terminalWriter.writeAsync(output);

        }

    }

}
