package at.qubic.shell.component.terminal;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.api.domain.std.SignedTransaction;
import at.qubic.api.domain.std.response.Entity;
import at.qubic.shell.component.TransactionContext;
import lombok.extern.slf4j.Slf4j;
import org.jline.terminal.Terminal;

import java.util.function.Supplier;

@Slf4j
public abstract class QxTerminalOutput extends AbstractTransactionTerminal<TransactionContext> {

    protected final String source;

    public QxTerminalOutput(IdentityUtil identityUtil, Supplier<Terminal> terminalSupplier, String source) {
        super(identityUtil, new TerminalWriter(terminalSupplier));
        this.source = source;
    }

    @Override
    public void writeSuccessMessage(TransactionContext tc) {
        String message = getResultMessage(tc);
        terminalWriter.writeAsync(message);
    }

    private String getResultMessage(TransactionContext tc) {
        Entity sourceBefore = tc.getSourceBefore();
        SignedTransaction stx = tc.getSignedTransaction();
        Entity sourceAfter = tc.getSourceAfter();
        String transactionInformation = getTransactionInformationString(stx);
        long outgoingAmount = sourceAfter.getOutgoingAmount() - sourceBefore.getOutgoingAmount();
        long balanceBefore = sourceBefore.getIncomingAmount() - sourceBefore.getOutgoingAmount();
        long balanceAfter = sourceAfter.getIncomingAmount() - sourceAfter.getOutgoingAmount();

        String balanceInformation = """
                    Balance change: %s
                    Latest outgoing tick: %d""".formatted(
                formatUtil.format(balanceAfter - balanceBefore),
                sourceAfter.getLatestOutgoingTransferTick());
        long expectedAmount = stx.getTransaction().getAmount();
        if (outgoingAmount >= expectedAmount) {

            log.info("Transaction [{}] successful.", stx.getTransactionHash());
            // \s to clear previous text
            return """
               Transaction successful. \s
              
               %s
               
               %s
               """.formatted(transactionInformation, balanceInformation);

        } else {
            // Balances did change but not enough. Transaction failed. This should never happen.
            String message = """
            Transaction included but NOT successful.
            
            %s
            
            Outgoing amount: %s
            %s
            """.formatted(transactionInformation,
                    formatUtil.formatUnsigned(outgoingAmount),
                    balanceInformation);
            log.error(message);
            return message;
        }
    }

}
