package at.qubic.shell.component.terminal;

import lombok.extern.slf4j.Slf4j;
import org.jline.terminal.Terminal;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;

import java.util.function.Supplier;

@Slf4j
public class TerminalWriter {

    private final Supplier<Terminal> terminalSupplier;

    public TerminalWriter(Supplier<Terminal> terminalSupplier) {
        this.terminalSupplier = terminalSupplier;
    }

    public void writeAsync(String message) {
        writePublisher(message).subscribe();
    }

    public Mono<?> writePublisher(String message) {
        return Mono.just(message)
                .publishOn(Schedulers.boundedElastic())
                .doOnNext(m -> {
                    log.debug("Terminal output: [{}].", m);
                    Terminal terminal = terminalSupplier.get();
                    terminal.writer().print(m);
                    terminal.flush();
                });
    }
}
