package at.qubic.shell.configuration;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.api.service.ComputorService;
import at.qubic.api.service.TransactionService;
import at.qubic.shell.command.*;
import at.qubic.shell.command.service.*;
import at.qubic.shell.completion.AssetIssuerCompletionProvider;
import at.qubic.shell.completion.AssetNameCompletionProvider;
import at.qubic.shell.completion.IdentityCompletionProvider;
import at.qubic.shell.component.*;
import at.qubic.shell.service.KeyManagementService;
import at.qubic.shell.service.SendQuTransferService;
import at.qubic.shell.service.SendTransactionService;
import at.qubic.shell.service.ShellComputorService;
import at.qubic.shell.wallet.Asset;
import at.qubic.shell.wallet.Assets;
import at.qubic.shell.wallet.KeyManager;
import at.qubic.shell.wallet.Wallets;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.shell.command.annotation.EnableCommand;

import java.time.Duration;

@EnableCommand({ NetworkCommands.class,
        EntityCommands.class,
        AddWalletCommand.class,
        ListWalletsCommand.class,
        RemoveWalletCommand.class,
        LockKeyStoreCommand.class,
        UpdatePasswordCommand.class,
        UnlockKeyStoreCommand.class,
        AddSeedCommand.class,
        RemoveSeedCommand.class,
        VerifyKeysCommand.class,
        SendTransactionCommand.class,
        QxFunctionsCommands.class,
        QxProcedureCommands.class,
        QearnFunctionCommands.class,
        QearnProcedureCommands.class
})
@Configuration
public class ShellConfiguration {

    @Bean
    EntityCommandService entityService(IdentityUtil identityUtil, ComputorService computorService) {
        return new EntityCommandService(computorService, identityUtil);
    }

    @Bean
    WalletService walletService(Wallets wallets, IdentityUtil identityUtil) {
        return new WalletService(wallets, identityUtil);
    }

    @Bean
    KeyService keyService(KeyManager keyManager, Wallets wallets, IdentityUtil identityUtil) {
        return new KeyService(keyManager, wallets, identityUtil);
    }

    @Bean
    KeyManagementService keyManagementService(KeyManager keyManager) {
        return new KeyManagementService(keyManager);
    }

    @Bean
    ShellComputorService shellComputorService(ComputorService computorService, IdentityUtil identityUtil) {
        return new ShellComputorService(computorService, identityUtil);
    }

    @Bean
    SendTransactionService sendTransactionService(ShellComputorService shellComputorService,
                                                  InclusionCheckComponent inclusionCheck) {
        return new SendTransactionService(shellComputorService, inclusionCheck);
    }

    @Bean
    SendQuTransferService sendQuTransferService(ShellComputorService computorService,
                                                InclusionCheckComponent inclusionCheck, IdentityUtil identityUtil) {
        return new SendQuTransferService(computorService, inclusionCheck, identityUtil);
    }

    @Bean(name = "identityCompletionProvider")
    IdentityCompletionProvider identityCompletionProvider(Wallets wallets) {
        return new IdentityCompletionProvider(wallets);
    }

    @Bean
    Assets assets(Environment environment) {
        String assetsKey = "known-assets";
        String[] knownAssets = environment.getRequiredProperty(assetsKey, String[].class);
        Assets assets = new Assets();
        for (String name : knownAssets) {
            String assetName = environment.getRequiredProperty(String.format("%s.%s.name", assetsKey, name));
            String assetIssuer = environment.getRequiredProperty(String.format("%s.%s.issuer", assetsKey, name));
            assets.add(Asset.builder().name(assetName).issuer(assetIssuer).build());
        }
        return assets;
    }

    @Bean
    AssetNameCompletionProvider assetNameCompletionProvider(Assets assets) {
        return new AssetNameCompletionProvider(assets);
    }

    @Bean
    AssetIssuerCompletionProvider assetIssuerCompletionProvider(Assets assets) {
        return new AssetIssuerCompletionProvider(assets);
    }

    @Bean
    PasswordComponent passwordComponent() {
        return new PasswordComponent();
    }

    @Bean
    InclusionCheck inclusionCheck(ComputorService computorService, Environment environment) {
        Duration checkInterval = environment.getRequiredProperty("transaction-check-interval", Duration.class);
        return new InclusionCheck(computorService, checkInterval);
    }

    @Bean
    InclusionCheckComponent inclusionCheckComponent(InclusionCheck inclusionCheck) {
        return new InclusionCheckComponent(inclusionCheck);
    }

    @Bean
    QuTransferComponent quTransferComponent(KeyManagementService keyManagementService, TransactionService transactionService,
                                            SendQuTransferService sendQuTransferService, IdentityUtil identityUtil) {
        return new QuTransferComponent(keyManagementService, transactionService, sendQuTransferService, identityUtil);
    }

    @Bean
    NetworkService networkService(ComputorService computorService) {
        return new NetworkService(computorService);
    }

}


