package at.qubic.shell.configuration;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.file.FileUtility;
import at.qubic.shell.repository.KeyStoreRepository;
import at.qubic.shell.repository.WalletFileStore;
import at.qubic.shell.repository.WalletRepository;
import at.qubic.shell.wallet.KeyManager;
import at.qubic.shell.wallet.Wallets;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;

@Configuration
public class WalletConfiguration {

    @Bean
    FileUtility fileUtility() {
        return new FileUtility();
    }

    @Bean
    KeyStoreRepository keyStoreRepository(FileUtility fileUtility, Environment environment) {
        final String directory = environment.getRequiredProperty("application.directory");
        final String file = environment.getRequiredProperty("application.keystore.file");
        final String saltFile = environment.getRequiredProperty("application.salt.file");
        return new KeyStoreRepository(fileUtility, directory, file, saltFile);
    }

    @Bean
    KeyManager keyManager(KeyStoreRepository keyStoreRepository) {
        return new KeyManager(keyStoreRepository);
    }

    @Bean
    WalletFileStore walletFileStore(FileUtility fileUtility, Environment environment) {
        final String directory = environment.getRequiredProperty("application.directory");
        final String file = environment.getRequiredProperty("application.wallet.file");
        return new WalletFileStore(fileUtility, directory, file);
    }

    @Bean
    WalletRepository walletRepository(WalletFileStore walletFileStore, IdentityUtil identityUtil) {
        return new WalletRepository(walletFileStore, identityUtil);
    }

    @Bean
    Wallets wallets(WalletRepository walletRepository) {
        Wallets wallets = new Wallets(walletRepository);
        wallets.loadWallets();
        return wallets;
    }

}
