package at.qubic.shell.repository;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.wallet.Wallet;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


@Slf4j
public class WalletRepository { // TODO refactor me. Combine with file store and lo

    private final WalletFileStore walletFileStore;
    private final IdentityUtil identityUtil;

    public WalletRepository(WalletFileStore walletFileStore, IdentityUtil identityUtil) {
        this.identityUtil = identityUtil;
        this.walletFileStore = walletFileStore;
    }

    public List<Wallet> load() {
        List<Wallet> wallets = new ArrayList<>();
        final List<String> ids = loadIdsFromFile();
        log.info("Loaded wallet with [{}] identities.", CollectionUtils.size(ids));
        for (String id : ids) {
            if (identityUtil.isValidIdentity(id)) {
                wallets.add(new Wallet(id));
            } else {
                log.warn("Skipping invalid identity [{}].", id);
            }
        }
        return wallets;
    }

    public void save(List<String> walletIds) throws IOException {
        if (!CollectionUtils.isEmpty(walletIds)) {
            walletFileStore.save(walletIds);
            log.info("Saved wallet.");
        }
    }

    private List<String> loadIdsFromFile() {
        try {
            return walletFileStore.load();
        } catch (IOException e) {
            log.error("Failed to load wallet identities from file.", e);
            return List.of(); // default to empty
        }
    }

}
