package at.qubic.shell.service;

import at.qubic.api.domain.std.SignedTransaction;
import at.qubic.api.domain.std.response.Entity;
import at.qubic.shell.component.InclusionCheckComponent;
import at.qubic.shell.component.TransactionContext;
import at.qubic.shell.exception.*;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

import java.util.function.Function;

@Slf4j
public class SendTransactionService extends AbstractSendTransactionService<TransactionContext> {


    public SendTransactionService(ShellComputorService computorService,
                                  InclusionCheckComponent inclusionCheck) {
        super(computorService, inclusionCheck);
    }

    protected Mono<TransactionContext> preProcessTransaction(String senderIdentity,
                                                           Function<Integer, SignedTransaction> createTransactionFunction,
                                                           boolean validate) {
        return computorService.getEntity(senderIdentity)
                .map(entity -> {
                    SignedTransaction stx = createTransactionFunction.apply(entity.getTick());
                    if (validate) {
                        verifyBalance(senderIdentity, entity, stx.getTransaction().getAmount());
                    }
                    return TransactionContext.builder()
                            .sourceBefore(entity)
                            .signedTransaction(stx)
                            .build();
                });
    }

    protected Mono<TransactionContext> postProcessTransaction(TransactionContext tc) {
        return computorService.getEntity(tc.getSourceBefore().getPublicKey(),
                        tc.getSignedTransaction().getTransaction().getTick() + 1)
                .map(entity -> {
                    tc.setSourceAfter(entity);
                    return tc;
                })
                .onErrorMap(e -> new PostProcessingException(tc, "Error postprocessing transaction.", e));
    }

    private void verifyBalance(String identity, Entity entity, long requiredBalance) {
        long balance = entity.getIncomingAmount() - entity.getOutgoingAmount();
        if (balance < requiredBalance) {
            String message = String.format("Insufficient balance on [%s]: [%d]. Required: [%d].",
                    identity, balance, requiredBalance);
            log.warn(message);
            throw new BeforSendingException(message);
        }
    }

}
