package at.qubic.shell.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.bouncycastle.crypto.DerivationParameters;
import org.bouncycastle.crypto.digests.Kangaroo;
import org.bouncycastle.crypto.generators.HKDFBytesGenerator;
import org.bouncycastle.crypto.generators.SCrypt;
import org.bouncycastle.crypto.params.HKDFParameters;
import org.springframework.security.crypto.encrypt.BytesEncryptor;
import org.springframework.security.crypto.encrypt.Encryptors;
import org.springframework.security.crypto.keygen.KeyGenerators;

import java.nio.CharBuffer;
import java.nio.charset.StandardCharsets;
import java.util.Base64;

@Slf4j
public class EncryptionUtility {

    public byte[] hashPassword(byte[] password, byte[] salt) {
        return SCrypt.generate(password, salt, 32768, 8, 3, 32);
    }

    public byte[] deriveKeyBase64(byte[] password, byte[] salt, String info) {
        final Kangaroo.KangarooTwelve kangarooTwelve = new Kangaroo.KangarooTwelve();
        final HKDFBytesGenerator generator = new HKDFBytesGenerator(kangarooTwelve);
        DerivationParameters derivationParameters = new HKDFParameters(password, salt, info.getBytes(StandardCharsets.UTF_8));
        generator.init(derivationParameters); // init and extract
        byte[] key = new byte[32];
        generator.generateBytes(key, 0, key.length); // expand
        return Base64.getEncoder().encode(key);
    }

    public byte[] generateSalt() {
        return KeyGenerators.secureRandom(16).generateKey();
    }

    public byte[] encryptData(char[] password, byte[] salt, byte[] data) {
        return encryptData(CharBuffer.wrap(password).asReadOnlyBuffer(), salt, data);
    }

    private byte[] encryptData(CharSequence password, byte[] salt, byte[] data) {
        BytesEncryptor encryptor = Encryptors.stronger(password, Hex.encodeHexString(salt));
        return encryptor.encrypt(data);
    }

    public byte[] decryptData(char[] password, byte[] salt, byte[] data) {
        return decryptData(CharBuffer.wrap(password).asReadOnlyBuffer(), salt, data);
    }

    private byte[] decryptData(CharSequence password, byte[] salt, byte[] data) {
        BytesEncryptor encryptor = Encryptors.stronger(password, Hex.encodeHexString(salt));
        return encryptor.decrypt(data);
    }

}
