package at.qubic.shell.util;

import java.math.BigInteger;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

public class FormatUtil {

    private final DecimalFormat amountFormatter;

    public FormatUtil() {
        amountFormatter = new DecimalFormat();
        DecimalFormatSymbols formatSymbols = DecimalFormatSymbols.getInstance();
        formatSymbols.setGroupingSeparator('\'');
        amountFormatter.setDecimalFormatSymbols(formatSymbols);
        amountFormatter.setGroupingUsed(true);
    }

    /**
     * Returns the formatted input as qu amount. The input is treated as a signed long. Negative values are
     * handled properly. For unsigned longs use the formatUnsigned method.
     * @param number The number to be formatted.
     * @return Then number formatted in 'qubic amount' sytle.
     */
    public String format(long number) {
        return amountFormatter.format(number);
    }

    /**
     * Returns the formatted input as qu amount. Format is feasible to display large numbers. Only suitable for unsigned
     * long values. Negative values will be shown as unsigned positive values.
     * Attention: not thread safe.
     * @param number the number to be formatted.
     * @return The number formatted in 'qubic amount' style.
     */
    public String formatUnsigned(long number) {
        BigInteger value = new BigInteger(Long.toUnsignedString(number));
        return amountFormatter.format(value);
    }

}
