package at.qubic.shell.wallet;

import at.qubic.shell.exception.UnknownAssetException;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;

@Slf4j
public class Assets {

    @Getter
    private final Set<Asset> assets = new HashSet<>();

    public void add(Asset asset) {
        log.info("Adding asset: {}", asset);
        assets.add(asset);
    }

    public Asset getUniqueAssetForName(String name) {
        List<Asset> assetsList = getAssetsForName(name);
        if (assetsList.isEmpty()) {
            String message = String.format(
                    "No asset found for asset name [%s]. Please specify name and issuer.", name);
            log.warn(message);
            throw new UnknownAssetException(message);
        } else if (assetsList.size() > 1) {
            String message = String.format(
                    "Multiple assets found for asset name [%s]. Please specify name and issuer.", name);
            log.warn(message);
            throw new UnknownAssetException(message);
        } else {
            Asset asset = assetsList.getFirst();
            log.info("Found asset: {}", asset);
            return asset;
        }
    }

    private List<Asset> getAssetsForName(String name) {
        List<Asset> filteredAssets = this.assets.stream().filter(a -> StringUtils.equals(name, a.getName())).toList();
        log.info("Found [{}] assets with name [{}].", filteredAssets.size(), name);
        return filteredAssets;

    }

    public Optional<Asset> getAsset(String name, String issuer) {
        return this.assets.stream()
                .filter(a -> StringUtils.equals(name, a.getName()))
                .filter(a -> StringUtils.equals(issuer, a.getIssuer()))
                .findAny();
    }

}
