package at.qubic.shell.wallet;

import jakarta.annotation.PreDestroy;
import lombok.extern.slf4j.Slf4j;
import at.qubic.shell.repository.WalletRepository;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class Wallets {

    private final WalletRepository walletRepository;
    private final Map<String, Wallet> wallets = new HashMap<>();

    public Wallets(WalletRepository walletRepository) {
        this.walletRepository = walletRepository;
    }

    public void loadWallets() {
        log.info("Loading wallets...");
        List<Wallet> walletList = walletRepository.load();
        for (Wallet wallet : walletList) {
            putWallet(wallet);
        }
    }

    @PreDestroy
    public void saveWallets() throws IOException {
        walletRepository.save(getWalletIds());
    }

    public void putWallet(Wallet wallet) {
        final Wallet old = wallets.put(wallet.identity(), wallet);
        if (old != null) {
            log.warn("Replaced wallet with identity [{}].", old.identity());
        } else {
            log.info("Added wallet with identity [{}].", wallet.identity());
        }
    }

    public boolean containsWalletIdentity(String identity) {
        return wallets.containsKey(identity);
    }

    public Wallet removeWallet(String identity) {
        return wallets.remove(identity);
    }

    public Wallet getWalletByIdentity(String identity) {
        return wallets.get(identity);
    }

    public List<String> getWalletIds() {
        return wallets.keySet().stream().sorted().toList();
    }

}
