package at.qubic.shell;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.shell.test.ShellAssertions;
import org.springframework.shell.test.ShellTestClient;
import org.springframework.shell.test.autoconfigure.ShellTest;

import java.time.Duration;

import static org.awaitility.Awaitility.await;

@ShellTest
public class QubicShellApplicationIT {

	@Autowired
	private ShellTestClient client;

	@Test
	void nonInteractiveShellStarts() {
		ShellTestClient.NonInteractiveShellSession session = client.nonInterative("help").run();
		await().atMost(Duration.ofSeconds(2)).untilAsserted(() -> ShellAssertions.assertThat(session.screen()).containsText("AVAILABLE COMMANDS"));
	}

	@Test
	void interactiveShellStarts() {
		ShellTestClient.InteractiveShellSession session = client.interactive().run();
		await().atMost(Duration.ofSeconds(2)).untilAsserted(() -> ShellAssertions.assertThat(session.screen()).containsText("shell"));
		session.write(session.writeSequence().text("help").carriageReturn().build());
		await().atMost(Duration.ofSeconds(2)).untilAsserted(() -> ShellAssertions.assertThat(session.screen()).containsText("AVAILABLE COMMANDS"));
	}

}
