package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.component.PasswordComponent;
import at.qubic.shell.command.service.KeyService;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

class AddSeedCommandTest {

    private final KeyService keyManagementService = mock();
    private final PasswordComponent passwordComponent = mock();
    private final AddSeedCommand command = new AddSeedCommand(keyManagementService, passwordComponent);

    @Test
    void addSeed_thenReturnIdentity() {
        when(passwordComponent.enterPassword(anyString(), anyString())).thenReturn("seed");
        when(keyManagementService.isValidSeed("seed")).thenReturn(true);
        when(keyManagementService.addSeed("seed")).thenReturn("some-identity");
        String result = command.addSeed();
        assertThat(result).contains("seed for identity [some-identity]");
    }

    @Test
    void addSeed_givenLocked_thenMessage() {
        when(keyManagementService.isLocked()).thenReturn(true);
        String result = command.addSeed();
        assertThat(result).contains("needs to be unlocked");
    }

    @Test
    void addSeed_givenInvalid_thenMessage() {
        String result = command.addSeed();
        assertThat(result).containsIgnoringCase("invalid seed");
        verify(keyManagementService, never()).addSeed(any());
    }

}