package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.command.service.WalletService;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class ListWalletsCommandTest {

    private final WalletService walletService = mock();
    private final ListWalletsCommand command = new ListWalletsCommand(walletService);

    @Test
    void listWallets() {
        when(walletService.getAllWalletIdentities()).thenReturn(List.of("a", "b", "c"));
        List<String> ids = command.listWallets(null);
        assertThat(ids).containsExactly("a", "b", "c");
    }

    @Test
    void listWallets_givenFilter_thenFilter() {
        when(walletService.getAllWalletIdentities()).thenReturn(List.of("TEST-A", "TEST-B", "FOO"));
        List<String> ids = command.listWallets("T.?ST.*");
        assertThat(ids).containsExactly("TEST-A", "TEST-B");
    }

    @Test
    void listWallets_givenInvalidFilterPattern_thenEmptyList() {
        assertThat(command.listWallets(")")).isEmpty();
    }

}