package at.qubic.shell.command;

import at.qubic.api.domain.qx.request.QxTransferAssetOrder;
import at.qubic.shell.exception.InputValidationException;
import org.junit.jupiter.api.Test;
import reactor.core.publisher.Mono;

import java.util.function.Function;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.when;

class QxTransferAssetCommandTest extends QxProcedureCommandsTest {

    protected Function<QxOrderArguments, String> executeProcedureCall() {
        return qoa -> commands.transferAsset(qoa.from(), "to", qoa.assetName(), qoa.issuer(), qoa.numberOfShares(), 100, qoa.offset(), qoa.retries(), qoa.validate());
    }

    @Test
    void order() throws InputValidationException {
        when(orderComponent.sendQxTransfer(eq("from"), any(QxTransferAssetOrder.class), eq(3), eq(0), eq(true))).thenReturn(Mono.empty());
        String result = executeProcedureCall().apply(defaultArguments());
        assertThat(result).isBlank();
    }

    @Test
    void order_givenInvalidToId_thenError() throws InputValidationException {
        doThrow(new InputValidationException("invalid destination")).when(orderComponent).validateIdentity("to");
        String result = executeProcedureCall().apply(defaultArguments());
        assertThat(result).isEqualTo("invalid destination");
    }

}