package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.command.service.KeyService;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

class RemoveSeedCommandTest {

    private final KeyService keyManagementService = mock();
    private final RemoveSeedCommand command = new RemoveSeedCommand(keyManagementService);

    @Test
    void removeSeed() {
        when(keyManagementService.isValidIdentity("foo")).thenReturn(true);
        when(keyManagementService.containsKeyForIdentity("foo")).thenReturn(true);
        assertThat(command.removeSeed("foo")).containsIgnoringCase("Deleted private keys");
        verify(keyManagementService).deletePrivateKey("foo");
    }

    @Test
    void removeSeed_givenLocked_thenMessage() {
        when(keyManagementService.isValidIdentity("foo")).thenReturn(true);
        when(keyManagementService.isLocked()).thenReturn(true);
        assertThat(command.removeSeed("foo")).containsIgnoringCase("locked");
    }

    @Test
    void removeSeed_givenUnknownKey_thenMessage() {
        when(keyManagementService.isValidIdentity("foo")).thenReturn(true);
        assertThat(command.removeSeed("foo")).containsIgnoringCase("No private keys stored");
    }

    @Test
    void removeSeed_givenInvalidKey_thenMessage() {
        assertThat(command.removeSeed("foo")).containsIgnoringCase("identity [foo] is invalid");
    }

}