package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.command.service.KeyService;
import at.qubic.shell.command.service.WalletService;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

class RemoveWalletCommandTest {

    private final WalletService walletService = mock();
    private final KeyService keyManagementService = mock();
    private final RemoveWalletCommand command = new RemoveWalletCommand(walletService, keyManagementService);

    @Test
    void removeWallet() {
        when(walletService.isValidIdentity("some-id")).thenReturn(true);
        when(walletService.removeWallet("some-id")).thenReturn("removed");
        assertThat(command.removeWallet("some-id")).isEqualTo("removed");
    }

    @Test
    void removeWallet_givenUnlockedKeyStore_thenDeleteFromKeyStore() {
        when(walletService.isValidIdentity("some-id")).thenReturn(true);
        when(keyManagementService.containsKeyForIdentity("some-id")).thenReturn(true);
        command.removeWallet("some-id");
        verify(keyManagementService).deletePrivateKey("some-id");
    }

    @Test
    void removeWallet_givenInvalidIdentity_thenMessage() {
        assertThat(command.removeWallet("foo")).containsIgnoringCase("identity [foo] is invalid");
    }

}