package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.component.PasswordComponent;
import at.qubic.shell.command.service.KeyService;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

class UnlockKeyStoreCommandTest {

    private final PasswordComponent passwordComponent = mock();
    private final KeyService keyManagementService = mock();
    private final UnlockKeyStoreCommand commands = new UnlockKeyStoreCommand(keyManagementService, passwordComponent);

    @Test
    void unlock() {
        when(keyManagementService.isLocked()).thenReturn(true);
        when(passwordComponent.enterPassword(anyString())).thenReturn("pwd");
        when(passwordComponent.isValidPassword("pwd")).thenReturn(true);
        when(keyManagementService.tryToUnlockKeyStore("pwd")).thenReturn("unlocked");
        assertThat(commands.unlock()).isEqualTo("unlocked");
    }

    @Test
    void unlock_givenAlreadyUnlocked_thenMessage() {
        assertThat(commands.unlock()).contains("already unlocked");
        verify(keyManagementService, never()).tryToUnlockKeyStore(anyString());
    }

    @Test
    void unlock_givenInvalidPassword_thenMessage() {
        when(keyManagementService.isLocked()).thenReturn(true);
        assertThat(commands.unlock()).contains("Invalid password");
        verify(keyManagementService, never()).tryToUnlockKeyStore(anyString());
    }

}