package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.component.PasswordComponent;
import at.qubic.shell.command.service.KeyService;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

class UpdatePasswordCommandTest {

    private final PasswordComponent passwordComponent = mock();
    private final KeyService keyManagementService = mock();
    private final UpdatePasswordCommand command = new UpdatePasswordCommand(keyManagementService, passwordComponent);

    @Test
    void updatePassword() {
        when(passwordComponent.enterPassword(anyString())).thenReturn("pwd");
        when(passwordComponent.isValidPassword("pwd")).thenReturn(true);
        when(keyManagementService.tryToUpdatePassword("pwd")).thenReturn("updated");
        assertThat(command.updatePassword()).isEqualTo("updated");
    }

    @Test
    void updatePassword_givenLocked_thenMessage() {
        when(keyManagementService.isLocked()).thenReturn(true);
        assertThat(command.updatePassword()).contains("Wallet needs to be unlocked");
    }

    @Test
    void updatePassword_givenInvalidPassword_thenMessage() {
        when(keyManagementService.tryToUpdatePassword("pwd")).thenReturn("updated");
        assertThat(command.updatePassword()).containsIgnoringCase("invalid password");
    }

    @Test
    void updatePassword_givenPasswordsDoNotMatch_thenMessage() {
        when(passwordComponent.enterPassword(anyString())).thenReturn("newpassword", "notmatching");
        assertThat(command.updatePassword()).containsIgnoringCase("passwords do not match");
    }

}