package at.qubic.shell.command;

import org.junit.jupiter.api.Test;
import at.qubic.shell.command.service.KeyService;
import at.qubic.shell.wallet.Wallets;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class VerifyKeysCommandTest {

    private final KeyService keyManagementService = mock();
    private final Wallets wallets = mock();
    private final VerifyKeysCommand command = new VerifyKeysCommand(keyManagementService, wallets);

    @Test
    void verify_givenValid_thenMessage() {
        when(keyManagementService.containsKeyForIdentity("some-identity")).thenReturn(true);
        when(keyManagementService.verifyIdentity("some-identity")).thenReturn(true);
        String result = command.verify("some-identity");
        assertThat(result).contains("key for identity [some-identity] is [valid].");
    }

    @Test
    void verify_givenInvalid_thenMessage() {
        when(keyManagementService.containsKeyForIdentity("some-identity")).thenReturn(true);
        String result = command.verify("some-identity");
        assertThat(result).contains("key for identity [some-identity] is [invalid].");
    }

    @Test
    void verify_givenLocked_thenMessage() {
        when(keyManagementService.isLocked()).thenReturn(true);
        String result = command.verify("some-identity");
        assertThat(result).containsIgnoringCase("wallet needs to be unlocked");
    }

    @Test
    void verify_givenNoKey_thenMessage() {
        String result = command.verify("some-identity");
        assertThat(result).contains("There is no key for identity [some-identity]");
    }

    @Test
    void verify_withoutIdentity_thenVerifyAll() {
        when(wallets.getWalletIds()).thenReturn(List.of("A", "B", "C"));
        when(keyManagementService.containsKeyForIdentity("A")).thenReturn(true);
        when(keyManagementService.containsKeyForIdentity("B")).thenReturn(true);
        when(keyManagementService.verifyIdentity("A")).thenReturn(true);

        String result = command.verify(null);

        assertThat(result).containsOnlyOnce("[A]: [valid]");
        assertThat(result).containsOnlyOnce("[B]: [invalid]");
        assertThat(result).doesNotContain("C");
    }

}