package at.qubic.shell.completion;

import org.junit.jupiter.api.Test;
import at.qubic.shell.wallet.Wallets;
import org.springframework.shell.CompletionProposal;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class IdentityCompletionProviderTest {

    private final Wallets wallets = mock();
    private final IdentityCompletionProvider  provider = new IdentityCompletionProvider(wallets);

    @Test
    void apply_thenReturnWalletIds() {
        when(wallets.getWalletIds()).thenReturn(List.of("a", "b", "c"));
        final List<CompletionProposal> proposals = provider.apply(mock());
        assertThat(proposals).hasSize(3);
        assertThat(proposals.stream().map(CompletionProposal::value)).containsExactly("a", "b", "c");
    }

    @Test
    void apply_givenNoWallets_thenReturnEmpty() {
        final List<CompletionProposal> proposals = provider.apply(mock());
        assertThat(proposals).isEmpty();
    }

}