package at.qubic.shell.component;

import at.qubic.api.crypto.IdentityUtil;
import at.qubic.api.service.TransactionService;
import at.qubic.shell.exception.InputValidationException;
import at.qubic.shell.service.KeyManagementService;
import at.qubic.shell.service.SendQuTransferService;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
class QuTransferComponentTest {

    private final KeyManagementService keyManagementService = mock();
    private final TransactionService transactionService = mock();
    private final SendQuTransferService sendQuTransferService = mock();
    private final IdentityUtil identityUtil = mock();
    private final QuTransferComponent quTransferComponent = new QuTransferComponent(keyManagementService, transactionService, sendQuTransferService, identityUtil);

    @Test
    void send_givenInvalidSourceIdentity_thenMessage() {
        when(identityUtil.isValidIdentity("from")).thenReturn(false);
        assertThatThrownBy(() -> quTransferComponent.transfer(42, "from",  "to", 3, 2, true).block())
                .isInstanceOf(InputValidationException.class);
    }

    @Test
    void send_givenInvalidTargetIdentity_thenMessage() {
        when(identityUtil.isValidIdentity("to")).thenReturn(false);
        assertThatThrownBy(() -> quTransferComponent.transfer(42, "from",  "to", 3, 2, true).block())
                .isInstanceOf(InputValidationException.class);
    }

    @Test
    void send_givenKeystoreLocked_thenMessage() {
        when(keyManagementService.isLocked()).thenReturn(true);
        assertThatThrownBy(() -> quTransferComponent.transfer(42, "from",  "to", 3, 2, true).block())
                .isInstanceOf(InputValidationException.class);
    }
  
}