package at.qubic.shell.component;

import at.qubic.shell.component.terminal.TerminalWriter;
import at.qubic.shell.component.terminal.DefaultTerminalErrorHandler;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import at.qubic.shell.exception.AfterSendingException;
import at.qubic.shell.exception.NotIncludedException;
import at.qubic.shell.exception.SendTransactionException;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;
import reactor.test.publisher.TestPublisher;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

class TransactionRetryHandlerTest {

    private final TestPublisher<String> testPublisher = TestPublisher.create();
    private final TerminalWriter terminalWriter = mock();
    private final DefaultTerminalErrorHandler retryHandler = new DefaultTerminalErrorHandler(terminalWriter);

    @BeforeEach
    void setUp() {
        when(terminalWriter.writePublisher(any())).thenReturn(Mono.empty());
    }

    @Test
    void retry_givenRetryableException_thenRetry() {
        StepVerifier.create(testPublisher.mono().retryWhen(retryHandler.retryIfRetryable(1)))
                .then(() -> testPublisher.error(new NotIncludedException("test")))
                .verifyErrorSatisfies(
                        e -> assertThat(e)
                                .isExactlyInstanceOf(SendTransactionException.class) // thrown after retries are exhausted
                                .hasMessageContaining("Retries exhausted")
                                .cause().isExactlyInstanceOf(NotIncludedException.class) // origin of retries
                );
    }

    @Test
    void retry_givenNoRetryableException_thenDoNotRetry() {
        StepVerifier.create(testPublisher.mono().retryWhen(retryHandler.retryIfRetryable(1)))
                .then(() -> testPublisher.error(new AfterSendingException("test")))
                .verifyErrorSatisfies(
                        e -> assertThat(e).hasNoCause()
                                .isExactlyInstanceOf(AfterSendingException.class)
                );
    }

}