package at.qubic.shell.repository;

import org.junit.jupiter.api.Test;
import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.wallet.Wallet;

import java.io.IOException;
import java.util.List;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

class WalletRepositoryTest {

    private final WalletFileStore walletFileStore = mock();
    private final IdentityUtil identityUtil = mock();
    private final WalletRepository walletRepository = new WalletRepository(walletFileStore, identityUtil);

    @Test
    void load() throws IOException {
        when(walletFileStore.load()).thenReturn(List.of("a", "b", "c"));
        when(identityUtil.isValidIdentity("a")).thenReturn(true);
        when(identityUtil.isValidIdentity("b")).thenReturn(true);
        List<Wallet> wallets = walletRepository.load();
        assertThat(wallets).hasSize(2);
        assertThat(wallets.stream().map(Wallet::identity)).containsExactly("a", "b");
    }

    @Test
    void load_givenNone_thenReturnEmpty() {
        List<Wallet> wallets = walletRepository.load();
        assertThat(wallets).isEmpty();
    }

    @Test
    void save_thenCallFileStore() throws IOException {
        List<String> ids = List.of("a", "b", "c");
        walletRepository.save(ids);
        verify(walletFileStore).save(ids);
    }


}