package at.qubic.shell.service;

import at.qubic.shell.command.service.QxService;
import org.junit.jupiter.api.Test;
import at.qubic.api.domain.qx.response.AssetOrder;
import at.qubic.api.domain.qx.response.EntityOrder;
import at.qubic.api.domain.qx.response.QxFees;
import at.qubic.api.service.ComputorService;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import java.util.List;

import static org.mockito.Mockito.*;

class QxServiceTest {

    private final ComputorService computorService = mock();
    private final QxService qxService = new QxService(computorService);

    @Test
    void getQxFees() {
        QxFees expected = QxFees.builder().build();
        when(computorService.getQxFees()).thenReturn(Mono.just(expected));
        StepVerifier.create(qxService.getQxFees())
                .expectNext(expected)
                .verifyComplete();
    }

    @Test
    void getAssetAskOrders() {
        List<AssetOrder> orders = List.of(AssetOrder.builder().build());
        when(computorService.getQxAskAssetOrders("issuer", "name", 0)).thenReturn(Mono.just(orders));
        StepVerifier.create(qxService.getAssetAskOrders("issuer", "name", 0))
                .expectNext(orders)
                .verifyComplete();
    }

    @Test
    void getAssetBidOrders() {
        List<AssetOrder> orders = List.of(AssetOrder.builder().build());
        when(computorService.getQxBidAssetOrders("issuer", "name", 0)).thenReturn(Mono.just(orders));
        StepVerifier.create(qxService.getAssetBidOrders("issuer", "name", 0))
                .expectNext(orders)
                .verifyComplete();
    }

    @Test
    void getEntityAskOrders() {
        List<EntityOrder> entityOrders = List.of(EntityOrder.builder().build());
        when(computorService.getQxAskEntityOrders("entity", 0)).thenReturn(Mono.just(entityOrders));
        StepVerifier.create(qxService.getEntityAskOrders("entity", 0))
                .expectNext(entityOrders)
                .verifyComplete();
    }

    @Test
    void getEntityBidOrders() {
        List<EntityOrder> entityOrders = List.of(EntityOrder.builder().build());
        when(computorService.getQxBidEntityOrders("entity", 0)).thenReturn(Mono.just(entityOrders));
        StepVerifier.create(qxService.getEntityBidOrders("entity", 0))
                .expectNext(entityOrders)
                .verifyComplete();
    }

}