package at.qubic.shell.service;

import at.qubic.shell.command.service.WalletService;
import org.junit.jupiter.api.Test;
import at.qubic.api.crypto.IdentityUtil;
import at.qubic.shell.wallet.Wallet;
import at.qubic.shell.wallet.Wallets;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

class WalletServiceTest {

    private final Wallets wallets = mock();
    private final IdentityUtil identityUtil = mock();
    private final WalletService walletService = new WalletService(wallets, identityUtil);

    @Test
    void addWallet_thenAddWallet() {
        when(identityUtil.isValidIdentity("some-id")).thenReturn(true);
        assertThat(walletService.addWallet("some-id")).contains("Added wallet");
        verify(wallets).putWallet(new Wallet("some-id"));
    }

    @Test
    void addWallet_givenInvalidId_thenDoNotAdd() {
        assertThat(walletService.addWallet("some-id")).contains("is invalid");
        verifyNoInteractions(wallets);
    }

    @Test
    void addWallet_givenAlreadyContains_thenDoNotAdd() {
        when(identityUtil.isValidIdentity("some-id")).thenReturn(true);
        when(wallets.containsWalletIdentity("some-id")).thenReturn(true);
        assertThat(walletService.addWallet("some-id")).contains("already a wallet");
        verify(wallets, never()).putWallet(any(Wallet.class));
    }

    @Test
    void getAllWalletIdentities_thenList() {
        when(wallets.getWalletIds()).thenReturn(List.of("a", "b", "c"));
        final List<String> ids = walletService.getAllWalletIdentities();
        assertThat(ids).containsExactly("a", "b", "c");
    }

    @Test
    void getAllWalletIdentities_givenEmpty_thenEmptyList() {
        final List<String> ids = walletService.getAllWalletIdentities();
        assertThat(ids).isEmpty();
    }

    @Test
    void removeWallet_givenNotFound_thenInform() {
        assertThat(walletService.removeWallet("some-id")).contains("There is no wallet");
    }

    @Test
    void removeWallet() {
        when(wallets.removeWallet("some-id")).thenReturn(new Wallet("foo"));
        assertThat(walletService.removeWallet("some-id")).contains("Removed wallet");
    }

}