package at.qubic.shell.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.junit.jupiter.api.Test;

import java.nio.charset.StandardCharsets;
import java.util.Base64;

import static org.assertj.core.api.Assertions.assertThat;

@Slf4j
class EncryptionUtilityTest {

    private final EncryptionUtility encryptionUtility = new EncryptionUtility();

    @Test
    void deriveKey() {
        byte[] password = "password".getBytes(StandardCharsets.UTF_8);
        byte[] salt = "salt".getBytes(StandardCharsets.UTF_8);
        assertThat(Hex.encodeHexString(encryptionUtility.deriveKeyBase64(password, salt, "info")))
                .isEqualTo("45464c78723946494775583978695534594849763557384161682b48726a73422f58397a776e6c4f35664d3d");
        assertThat(Hex.encodeHexString(encryptionUtility.deriveKeyBase64(password, salt, "another")))
                .isEqualTo("5a62442f6778754c75373337576843585274667a444e444c4e4857536361504f66576773512b67505747733d");
    }

    @Test
    void hashPassword() {
        byte[] salt = Base64.getDecoder().decode("DCPyuN1dPoDGUPdLcRJ8xe");
        byte[] hashed = encryptionUtility.hashPassword("password".getBytes(StandardCharsets.UTF_8), salt);
        assertThat(Hex.encodeHexString(hashed)).isEqualTo("d6ace3179e036156ab2126ec9b6763646ed9677649e3c28e8e5e818d7025d25d");
    }

    // test to play with hkdf generator
    @Test
    void generateSalt() {
        byte[] bytes = encryptionUtility.generateSalt();
        assertThat(bytes.length).isEqualTo(16);
        assertThat(bytes).isNotEqualTo(encryptionUtility.generateSalt());
    }

    @Test
    void encryptData() {
        byte[] encrypted = encryptionUtility.encryptData("password".toCharArray(), new byte[] { 1, 2, 3, 4 }, "test".getBytes(StandardCharsets.UTF_8));
        assertThat(encrypted).hasSize(36); // size depends on data
        log.info(Hex.encodeHexString(encrypted));
    }

    @Test
    void decryptData() throws Exception {
        byte[] decrypted = encryptionUtility.decryptData("password".toCharArray(), new byte[] { 1, 2, 3, 4 }, Hex.decodeHex("747d8043b04b46ead1c14eb0c0a7ead3ecb288eea95d15e5f0e1762cdfe4ec1dc8c95e38"));
        assertThat(decrypted).isEqualTo("test".getBytes(StandardCharsets.UTF_8));
    }

    @Test
    void encryptAndDecryptData() {
        byte[] data = "test".getBytes(StandardCharsets.UTF_8);
        byte[] encrypted = encryptionUtility.encryptData("password".toCharArray(), new byte[] { 1, 2, 3, 4 }, data);
        byte[] decrypted = encryptionUtility.decryptData("password".toCharArray(), new byte[] { 1, 2, 3, 4 }, encrypted);
        assertThat(decrypted).isEqualTo(data);
    }

}