package at.qubic.shell.wallet;

import org.junit.jupiter.api.Test;
import at.qubic.shell.repository.WalletRepository;

import java.io.IOException;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

class WalletsTest {

    private final WalletRepository walletRepository = mock();
    private final Wallets wallets = new Wallets(walletRepository);

    @Test
    void putWallet_thenReturnOnGet() {
        Wallet w = new Wallet("some-id");
        wallets.putWallet(w);
        assertThat(wallets.getWalletByIdentity("some-id")).isEqualTo(w);
    }

    @Test
    void putWallet_thenReplace() {
        Wallet w1 = new Wallet("some-id");
        Wallet w2 = new Wallet("some-id");
        wallets.putWallet(w1);
        wallets.putWallet(w2);
        final Wallet retrieved = wallets.getWalletByIdentity("some-id");
        assertThat(retrieved).isSameAs(w2);
        assertThat(retrieved).isNotSameAs(w1);
        assertThat(wallets.getWalletIds()).containsExactly("some-id");
    }

    @Test
    void getWallet_unknownId_thenNull() {
        assertThat(wallets.getWalletByIdentity("foo")).isNull();
    }

    @Test
    void getWalletIds() {
        Wallet w1 = new Wallet("some-id");
        Wallet w2 = new Wallet("another-id");
        wallets.putWallet(w1);
        wallets.putWallet(w2);
        assertThat(wallets.getWalletIds()).containsExactly("another-id", "some-id");
    }

    @Test
    void containsWallet() {
        wallets.putWallet(new Wallet("hello"));
        assertThat(wallets.containsWalletIdentity("some-id")).isFalse();
        assertThat(wallets.containsWalletIdentity("hello")).isTrue();
    }

    @Test
    void removeWallet() {
        Wallet wallet = new Wallet("hello");
        wallets.putWallet(wallet);
        assertThat(wallets.removeWallet("some-id")).isNull();
        assertThat(wallets.removeWallet("hello")).isEqualTo(wallet);
        assertThat(wallets.containsWalletIdentity("hello")).isFalse();
    }

    @Test
    void loadWallets() {
        when(walletRepository.load()).thenReturn(List.of(new Wallet("a"), new Wallet("b"), new Wallet("b")));
        wallets.loadWallets();
        assertThat(wallets.getWalletIds()).containsExactly("a", "b");
    }

    @Test
    void saveWallets() throws IOException {
        wallets.putWallet(new Wallet("a"));
        wallets.putWallet(new Wallet("b"));
        wallets.putWallet(new Wallet("c"));
        wallets.saveWallets();
        verify(walletRepository).save(List.of("a", "b", "c"));
    }

}