# qubicj-vanity-id

Contains a tool for generating vanity IDs.

## Getting started

Follow the [build instructions in the parent project](../README.md#build).

The application is created in the submodules `target/` directory.

It is recommened to use the standard fourQ implementation instead of the qubic version as it is faster. See [below](#shared-library).

## Run

Execute the jar files and provide the required and optional options (for more options see source code).

```
Usage: java -jar ... --vanity=<list,of,names> [--iterations=<no-of-iterations>] [--cores=<no-of-used-cores>] [--matchStart] [--matchEnd]

vanity ... list of strings the ID should start with.
iterations ... number of iterations until the program finishes. You can always stop the application with CTRL+C.
matchStart (optional) ... match strings at the beginning of the ID (default). Can be used together with matchEnd.
matchEnd (optional) ... match strings at the end of the ID. Can be used together with matchStart.
cores (optional) ... number of cores (virtual cores, threads) the app should use for parallel processing.

```
Don't use too many cores or performance will degrade over time. The default should be fine. 
Make sure to remove the log files from your system for security reasons in case you want to use one of the generated seeds.
By default, the logs are written to `vanity.log`.

## Shared library

There are two options: either use the default qubic fourq implementation (see [qubicj-computor-api](../qubicj-computor-api/README.md)) or use
the approx 25% faster [FourQ](https://github.com/microsoft/FourQlib) standard fourQ library (additional shared library installation necessary).

### Standard FourQ Library (optional but faster)

You need to install the shared library:

1. Clone [repository](https://github.com/microsoft/FourQlib)
2. Compile
3. Install shared library
4. Configure property

#### Compile

For Linux (amd64) go into `FourQ_64bit_and_portable` folder and follow the instructions in [README.md](https://github.com/microsoft/FourQlib/blob/master/FourQ_64bit_and_portable/README.md). It's important to use
the optimizations that are available on your target machine for optimal performance. I used the following make command:

```shell
cd FourQ_64bit_and_portable
make ARCH=x64 CC=clang ASM=TRUE AVX=TRUE AVX2=TRUE EXTENDED_SET=TRUE USE_ENDO=TRUE SHARED_LIB=TRUE
```

#### Install

Do not forget to run the tests of the fourq library and verify that all are passing as some compile options lead to invalid binaries.

Install the library with:

```shell
sudo cp libFourQ.so /usr/local/lib
sudo ldconfig
```

#### Configure

Set property in `application.properties` to use the standard implementation: 

```properties
fourq.lib.name=FourQ
```

## Performance

On my Ryzen-5950X (16 cores, 32 threads, 64GB RAM) with 15 threads and the standard fourQ implementation one iteration needs about 1000 nanoseconds. 
That means 100_000_000 iterations take about 100 seconds. Time needed with 1000000 it/s needed (average):

* 4 letters -> 26^4 -> 456976 iterations -> about half a second
* 5 letters -> 26^5 -> 11881376 iterations -> about ten seconds
* 6 letters -> 26^6 -> 308915776 iterations -> more than five minutes
* 7 letters -> 26^7 -> 8031810176 iterations -> more than two hours

## Usage Examples

Look for strings starting with 'AAAAA' or 'BBBBB', do 10000000 iterations and use 10 cores:

```shell
java -jar target/qubicj-vanity-id-0.0.1-SNAPSHOT.jar --vanity=AAAAA,BBBBB --iterations=10000000 --cores=10
```

Complete example. Uses property `fourq.lib.name=FourQ` for better performance. Generated three seeds for ID starting with 'AAAAA':

```shell
java -jar target/qubicj-vanity-id-1.0.0-SNAPSHOT.jar --vanity=AAAAA,BBBBB --iterations=10000000

  .   ____          _            __ _ _
 /\\ / ___'_ __ _ _(_)_ __  __ _ \ \ \ \
( ( )\___ | '_ | '_| | '_ \/ _` | \ \ \ \
 \\/  ___)| |_)| | | | | || (_| |  ) ) ) )
  '  |____| .__|_| |_|_| |_\__, | / / / /
 =========|_|==============|___/=/_/_/_/

 :: Spring Boot ::                (v3.3.1)

2024-07-17T08:32:55.757Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Starting QubicVanityIdApplication v1.0.0-SNAPSHOT using Java 21.0.4-ea with PID 53010 (/home/mio/prog/projects/qubic/qubicj/qubicj-vanity-id/target/qubicj-vanity-id-1.0.0-SNAPSHOT.jar started by mio in /home/mio/prog/projects/qubic/qubicj/qubicj-vanity-id)
2024-07-17T08:32:55.758Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : The following 1 profile is active: "local"
2024-07-17T08:32:56.064Z  INFO 53010 --- [           main] o.mio.qubic.computor.crypto.QubicFourQ   : Using fourq library: [FourQ].
2024-07-17T08:32:56.208Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Started QubicVanityIdApplication in 0.789 seconds (process running for 1.035)
2024-07-17T08:32:56.210Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Estimated needed number of iterations for search string [AAAAA]: [11881376].
2024-07-17T08:32:56.210Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Estimated needed number of iterations for search string [BBBBB]: [11881376].
2024-07-17T08:32:56.210Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Generating vanity IDs starting with the following [2] strings: [AAAAA, BBBBB].
2024-07-17T08:32:56.210Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Maximum number of iterations: [10000000].
2024-07-17T08:32:56.210Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Using [15] cores.
2024-07-17T08:32:56.210Z  INFO 53010 --- [           main] o.m.q.vanity.QubicVanityIdApplication    : Started at [2024-07-17T08:32:56.210390458].
2024-07-17T08:32:57.270Z  INFO 53010 --- [       single-1] o.m.q.vanity.QubicVanityIdApplication    : [ioqbokocxwtfmcobjyopqnrjkxdpcfeqcctlvgameqvorgpkbkgufyv] => [AAAAABVWOWERBDTTGJWZWGKEVAJAJXBBPWQOVFDLHCPPWFBRDIZIZFSGQQRN]
2024-07-17T08:33:01.905Z  INFO 53010 --- [       single-1] o.m.q.vanity.QubicVanityIdApplication    : [pikcieywicanewbqbmhaphjeqqtwvbtstfdzupplrjzxobvtosmbqll] => [AAAAAPAEMEWNHCPWNWIWKTIOLYRBDKQNQZECOZEWPDGJDHKXESNODDKBZWEK]
2024-07-17T08:33:05.301Z  INFO 53010 --- [       single-1] o.m.q.vanity.QubicVanityIdApplication    : [ftdksxcluullwnitnpyfbckvuvrssojvcokpoqabvcucjsnsmueeakn] => [AAAAALGRRBGDIFRIUCEKGJNDPFCBWFMTZSSVGERCVBBPCIRQQSVSOXMGQZOD]
2024-07-17T08:33:07.286Z  INFO 53010 --- [    parallel-10] o.m.q.vanity.keygen.VanityIdGenerator    : [10000000] iterations. Current speed: 905633 it/s
2024-07-17T08:33:07.288Z  INFO 53010 --- [       single-1] o.m.q.vanity.QubicVanityIdApplication    : End at [2024-07-17T08:33:07.288427904]
2024-07-17T08:33:07.288Z  INFO 53010 --- [       single-1] o.m.q.vanity.QubicVanityIdApplication    : Iterations: [10000000]. Duration: 11s. 1107ns/it
2024-07-17T08:33:07.288Z  INFO 53010 --- [       single-1] o.m.q.vanity.QubicVanityIdApplication    : Completed.
```
