package at.qubic.vanity;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import at.qubic.api.crypto.FourQ;
import at.qubic.api.crypto.QubicFourQ;
import at.qubic.vanity.crypto.IdentityUtil;
import at.qubic.vanity.crypto.RandomGeneratorFactory;
import at.qubic.vanity.keygen.IdFragmentGenerator;
import at.qubic.vanity.keygen.VanityIdGenerator;
import org.springframework.boot.context.properties.ConfigurationPropertiesScan;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;

@Configuration
@ConfigurationPropertiesScan
@Slf4j
public class ApplicationConfig {

    @Bean
    FourQ fourQ(Environment env) {
        String fourQ = env.getRequiredProperty("fourq.lib.name");
        return new QubicFourQ(fourQ);
    }

    @Bean
    IdentityUtil identityUtil(FourQ fourQ) {
        return new IdentityUtil(fourQ);
    }

    @Bean
    RandomGeneratorFactory randomGeneratorFactory() {
        return new RandomGeneratorFactory();
    }

    @Bean
    VanityIdGenerator vanityKeyGenerator(RandomGeneratorFactory randomGeneratorFactory, FourQ fourQ, Environment environment) {
        final String seedFragment = environment.getProperty("seedFragment");
        if (StringUtils.isNotBlank(seedFragment)) {
            log.info("Creating seed with fragment [{}].", seedFragment);
            return new IdFragmentGenerator(seedFragment, randomGeneratorFactory, fourQ);
        } else {
            return new VanityIdGenerator(randomGeneratorFactory, fourQ);
        }
    }

}
