package at.qubic.vanity.crypto;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.rng.UniformRandomProvider;
import org.apache.commons.text.RandomStringGenerator;

import java.security.SecureRandom;
import java.util.function.Supplier;

@Slf4j
public class RandomGeneratorFactory {

    final Supplier<UniformRandomProvider> randomProviderSupplier;

    public RandomGeneratorFactory(Supplier<UniformRandomProvider> randomProviderSupplier) {
        this.randomProviderSupplier = randomProviderSupplier;
    }

    public RandomGeneratorFactory() {
            this.randomProviderSupplier = this::secureProvider;
    }

    public RandomStringGenerator create() {
        final UniformRandomProvider randomProvider = randomProviderSupplier.get();
        return new RandomStringGenerator.Builder()
                .withinRange('a', 'z')
                .usingRandom(randomProvider::nextInt)
                .build();
    }

    // good and secure default but slow
    private UniformRandomProvider secureProvider() {
        log.debug("Secure random provider.");
        SecureRandom secureRandom = new SecureRandom();
        return secureRandom::nextLong;
    }

}
