package at.qubic.vanity.crypto;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.text.RandomStringGenerator;
import org.springframework.util.Assert;

public class SeedFragmentGenerator implements SeedGenerator {

    private static final int SEED_LENGTH = 55;

    private final String fixSeedFragment;
    private final int generatedPartSize;
    private int shift = Integer.MAX_VALUE;
    private String generatedSeed = null;
    private final RandomStringGenerator randomStringGenerator;

    /**
     * Generator for seed generation based on a seed fragment. The fragment will be part of the generated seed. The seed fragment has to be at the beginning of the result seed.
     * @param seedFragment The fragment the generated seed begins with. Must not be longer than 54 characters.
     * @param randomStringGenerator generator to be used for random string parts.
     */
    public SeedFragmentGenerator(String seedFragment, RandomStringGenerator randomStringGenerator) {
        Assert.isTrue(StringUtils.containsOnly(seedFragment, "abcdefghijklmnopqrstuvwxyz"), String.format("Invalid seed fragment character: [%s]", seedFragment));
        this.fixSeedFragment = seedFragment;
        this.generatedPartSize = SEED_LENGTH - StringUtils.length(seedFragment);
        Assert.isTrue(generatedPartSize > 0, String.format("Invalid seed fragment size [%s]: [%s]", StringUtils.length(seedFragment), seedFragment));
        this.randomStringGenerator = randomStringGenerator;
    }

    // careful not thread safe
    public String seed() {
        if (shift < generatedPartSize - 1) { // secure random is slow. rotate string.
            shift++;
            return fixSeedFragment + StringUtils.rotate(generatedSeed, shift);
        } else {
            generatedSeed = randomStringGenerator.generate(generatedPartSize);
            shift = 0;
            return fixSeedFragment + generatedSeed;
        }
    }
}
