package at.qubic.vanity.crypto;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.text.RandomStringGenerator;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.*;

@Slf4j
class StandardSeedGeneratorTest {

    private final RandomStringGenerator randomStringGenerator = mock();
    private final StandardSeedGenerator seedGenerator = new StandardSeedGenerator(randomStringGenerator);

    @Test
    void generate_thenStartWithFragment() {
        when(randomStringGenerator.generate(55)).thenReturn("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123");
        assertThat(seedGenerator.seed()).isEqualTo("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123");
    }

    @Test
    void generate_thenRotateString() {
        String seed = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123";
        when(randomStringGenerator.generate(55)).thenReturn(seed);
        for (int i = 0; i < 55; i++) {
            assertThat(seedGenerator.seed()).isEqualTo(StringUtils.rotate(seed, i));
        }
    }

}