package at.qubic.vanity.keygen;

import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.time.Duration;
import java.time.LocalDateTime;

@Slf4j
@SpringBootTest
class VanityIdGeneratorIT {

    @Autowired
    private VanityIdGenerator keyGenerator;

    @Test // only starts key generator, does not really verify much
    public void generateSeeds() {

        long iterations = 10;
        long start = System.nanoTime();
        int cores = Math.max(2, (Runtime.getRuntime().availableProcessors() / 2) - 1);
        log.info("Using [{}] cores for parallel processing.", cores);
        log.info("Start at [{}].", LocalDateTime.now());

        keyGenerator.generate(cores, iterations, true, false, "ABCDEFG")
                .doOnNext(match -> log.info("[{}] => [{}]", match.getT1(), match.getT2()))
                .doOnError((t -> log.error(t.toString(), t)))
                .blockLast();

        long duration = System.nanoTime() - start;
        Duration d = Duration.ofNanos(duration);
        log.info("End at [{}]", LocalDateTime.now());
        log.info("Iterations: [{}]. Duration: {}s. {}ns/it", iterations, d.toSeconds(), duration/iterations);
    }

}