package at.qubic.vanity.keygen;

import org.apache.commons.rng.UniformRandomProvider;
import org.apache.commons.rng.core.source64.DotyHumphreySmallFastCounting64;
import org.junit.jupiter.api.Test;
import at.qubic.api.crypto.QubicFourQ;
import at.qubic.vanity.crypto.RandomGeneratorFactory;
import reactor.test.StepVerifier;

import static org.assertj.core.api.Assertions.assertThat;

class VanityIdGeneratorTest {

    private final UniformRandomProvider uniformRandomProvider = new DotyHumphreySmallFastCounting64(new long[] {1,2,3});
    private final VanityIdGenerator keyGenerator = new VanityIdGenerator(new RandomGeneratorFactory(() -> uniformRandomProvider), new QubicFourQ());

    // ICPISJZSILPUTAEGECBAHBKFUKGCTFZYDVBIMZSPXEXNDWPAPAACYZYAWFCJ
    // CRJJXGSKVWRKKDSJQPKKHOGYJLEDJQEEYWQIVHHVWFVPZJRJEYGSUEJABWWC
    // TJNXEJUJFTVWQCFZPBQBMJDCFSMCLFECCTAJXSZGHERVELPDTBWFENFGFXFB

    @Test
    void generate_matchStart() {

        StepVerifier.create(keyGenerator.generate(1, 2, true, false, "ICPIS"))
                .assertNext(t -> assertThat(t.getT2().toString()).isEqualTo("ICPISJZSILPUTAEGECBAHBKFUKGCTFZYDVBIMZSPXEXNDWPAPAACYZYAWFCJ"))
                .verifyComplete();
    }

    @Test
    void generate_matchEnd() {

        StepVerifier.create(keyGenerator.generate(1, 2, false, true, "ABWWC"))
                .assertNext(t -> assertThat(t.getT2().toString()).isEqualTo("CRJJXGSKVWRKKDSJQPKKHOGYJLEDJQEEYWQIVHHVWFVPZJRJEYGSUEJABWWC"))
                .verifyComplete();
    }

    @Test
    void generate_matchStartAndEnd() {

        StepVerifier.create(keyGenerator.generate(1, 3, true, true, "I", "C", "T"))
                .expectNextCount(3)
                .verifyComplete();
    }

    @Test
    void generate_correctSeedLength() {

        StepVerifier.create(keyGenerator.generate(1, 1, true, false, "ICPIS"))
                .assertNext(t -> assertThat(t.getT1()).hasSize(55).isLowerCase())
                .verifyComplete();

    }

}